# sage_setup: distribution = sagemath-combinat
r"""
Examples of Lie Conformal Algebras

We implement the following examples of Lie conformal algebras:

- :mod:`Abelian Lie conformal algebra<.abelian_lie_conformal_algebra>`
- :mod:`Affine Lie conformal algebra<.affine_lie_conformal_algebra>`
- :mod:`Bosonic Ghosts<.bosonic_ghosts_lie_conformal_algebra>`
- :mod:`Fermionic Ghosts<.fermionic_ghosts_lie_conformal_algebra>`
- :mod:`Free Bosons<.free_bosons_lie_conformal_algebra>`
- :mod:`Free Fermions<.free_fermions_lie_conformal_algebra>`
- :mod:`N=2 super Lie Conformal algebra<.n2_lie_conformal_algebra>`
- :mod:`Neveu-Schwarz super Lie conformal
  algebra<.neveu_schwarz_lie_conformal_algebra>`
- :mod:`Virasoro Lie conformal algebra<.virasoro_lie_conformal_algebra>`
- :mod:`Weyl Lie conformal algebra<.weyl_lie_conformal_algebra>`

AUTHORS:

- Reimundo Heluani (2020-06-15): Initial implementation.
"""

# *****************************************************************************
#       Copyright (C) 2020 Reimundo Heluani <heluani@potuz.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************

from .abelian_lie_conformal_algebra import AbelianLieConformalAlgebra as Abelian
from .affine_lie_conformal_algebra import AffineLieConformalAlgebra as Affine
from .bosonic_ghosts_lie_conformal_algebra import BosonicGhostsLieConformalAlgebra as BosonicGhosts
from .fermionic_ghosts_lie_conformal_algebra import FermionicGhostsLieConformalAlgebra as FermionicGhosts
from .free_bosons_lie_conformal_algebra import FreeBosonsLieConformalAlgebra as FreeBosons
from .free_fermions_lie_conformal_algebra import FreeFermionsLieConformalAlgebra as FreeFermions
from .n2_lie_conformal_algebra import N2LieConformalAlgebra as N2
from .neveu_schwarz_lie_conformal_algebra import NeveuSchwarzLieConformalAlgebra as NeveuSchwarz
from .virasoro_lie_conformal_algebra import VirasoroLieConformalAlgebra as Virasoro
from .weyl_lie_conformal_algebra import WeylLieConformalAlgebra as Weyl



# sage_setup: distribution = sagemath-combinat
"""
Subset species
"""

# ****************************************************************************
#       Copyright (C) 2008 Mike Hansen <mhansen@gmail.com>,
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#    This code is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    General Public License for more details.
#
#  The full text of the GPL is available at:
#
#                  http://www.gnu.org/licenses/
# ****************************************************************************

from .species import GenericCombinatorialSpecies
from .set_species import SetSpecies
from .structure import GenericSpeciesStructure
from sage.combinat.species.misc import accept_size
from sage.structure.unique_representation import UniqueRepresentation
from sage.arith.misc import factorial


class SubsetSpeciesStructure(GenericSpeciesStructure):
    def __repr__(self):
        """
        EXAMPLES::

            sage: set_random_seed(0)
            sage: S = species.SubsetSpecies()
            sage: a = S.structures(["a","b","c"])[0]; a
            {}
        """
        s = GenericSpeciesStructure.__repr__(self)
        return "{"+s[1:-1]+"}"

    def canonical_label(self):
        """
        Return the canonical label of ``self``.

        EXAMPLES::

            sage: P = species.SubsetSpecies()
            sage: S = P.structures(["a", "b", "c"])
            sage: [s.canonical_label() for s in S]
            [{}, {'a'}, {'a'}, {'a'}, {'a', 'b'}, {'a', 'b'}, {'a', 'b'}, {'a', 'b', 'c'}]
        """
        rng = list(range(1, len(self._list) + 1))
        return self.__class__(self.parent(), self._labels, rng)

    def label_subset(self):
        r"""
        Return a subset of the labels that "appear" in this structure.

        EXAMPLES::

            sage: P = species.SubsetSpecies()
            sage: S = P.structures(["a", "b", "c"])
            sage: [s.label_subset() for s in S]
            [[], ['a'], ['b'], ['c'], ['a', 'b'], ['a', 'c'], ['b', 'c'], ['a', 'b', 'c']]
        """
        return [self._relabel(i) for i in self._list]

    def transport(self, perm):
        r"""
        Return the transport of this subset along the permutation perm.

        EXAMPLES::

            sage: F = species.SubsetSpecies()
            sage: a = F.structures(["a", "b", "c"])[5]; a
            {'a', 'c'}
            sage: p = PermutationGroupElement((1,2))                                    # needs sage.groups
            sage: a.transport(p)                                                        # needs sage.groups
            {'b', 'c'}
            sage: p = PermutationGroupElement((1,3))                                    # needs sage.groups
            sage: a.transport(p)                                                        # needs sage.groups
            {'a', 'c'}
        """
        l = sorted([perm(i) for i in self._list])
        return SubsetSpeciesStructure(self.parent(), self._labels, l)

    def automorphism_group(self):
        r"""
        Return the group of permutations whose action on this subset leave
        it fixed.

        EXAMPLES::

            sage: F = species.SubsetSpecies()
            sage: a = F.structures([1,2,3,4])[6]; a
            {1, 3}
            sage: a.automorphism_group()                                                # needs sage.groups
            Permutation Group with generators [(2,4), (1,3)]

        ::

            sage: [a.transport(g) for g in a.automorphism_group()]                      # needs sage.groups
            [{1, 3}, {1, 3}, {1, 3}, {1, 3}]
        """
        from sage.groups.perm_gps.permgroup_named import SymmetricGroup
        from sage.groups.perm_gps.permgroup import PermutationGroup
        a = SymmetricGroup(self._list)
        b = SymmetricGroup(self.complement()._list)
        return PermutationGroup(a.gens() + b.gens())

    def complement(self):
        r"""
        Return the complement of ``self``.

        EXAMPLES::

            sage: F = species.SubsetSpecies()
            sage: a = F.structures(["a", "b", "c"])[5]; a
            {'a', 'c'}
            sage: a.complement()
            {'b'}
        """
        new_list = [i for i in range(1, len(self._labels)+1) if i not in self._list]
        return SubsetSpeciesStructure(self.parent(), self._labels, new_list)


class SubsetSpecies(GenericCombinatorialSpecies, UniqueRepresentation):
    @staticmethod
    @accept_size
    def __classcall__(cls, *args, **kwds):
        """
        EXAMPLES::

            sage: S = species.SubsetSpecies(); S
            Subset species
        """
        return super().__classcall__(cls, *args, **kwds)

    def __init__(self, min=None, max=None, weight=None):
        """
        Return the species of subsets.

        EXAMPLES::

            sage: S = species.SubsetSpecies()
            sage: S.generating_series()[0:5]
            [1, 2, 2, 4/3, 2/3]
            sage: S.isotype_generating_series()[0:5]
            [1, 2, 3, 4, 5]

            sage: S = species.SubsetSpecies()
            sage: c = S.generating_series()[0:3]
            sage: S._check()
            True
            sage: S == loads(dumps(S))
            True
        """
        GenericCombinatorialSpecies.__init__(self, min=None, max=None, weight=None)
        self._name = "Subset species"

    _default_structure_class = SubsetSpeciesStructure

    def _structures(self, structure_class, labels):
        """
        EXAMPLES::

            sage: S = species.SubsetSpecies()
            sage: S.structures([1,2]).list()
            [{}, {1}, {2}, {1, 2}]
            sage: S.structures(['a','b']).list()
            [{}, {'a'}, {'b'}, {'a', 'b'}]
        """
        from sage.combinat.combination import Combinations
        for c in Combinations(range(1, len(labels)+1)):
            yield structure_class(self, labels, c)

    def _isotypes(self, structure_class, labels):
        """
        EXAMPLES::

            sage: S = species.SubsetSpecies()
            sage: S.isotypes([1,2]).list()
            [{}, {1}, {1, 2}]
            sage: S.isotypes(['a','b']).list()
            [{}, {'a'}, {'a', 'b'}]
        """
        for i in range(len(labels)+1):
            yield structure_class(self, labels, range(1, i+1))

    def _gs_callable(self, base_ring, n):
        """
        The generating series for the species of subsets is
        `e^{2x}`.

        EXAMPLES::

            sage: S = species.SubsetSpecies()
            sage: [S.generating_series().coefficient(i) for i in range(5)]
            [1, 2, 2, 4/3, 2/3]
        """
        return base_ring(2)**n / base_ring(factorial(n))

    def _itgs_callable(self, base_ring, n):
        r"""
        The generating series for the species of subsets is
        `e^{2x}`.

        EXAMPLES::

            sage: S = species.SubsetSpecies()
            sage: S.isotype_generating_series()[0:5]
            [1, 2, 3, 4, 5]
        """
        return base_ring(n + 1)

    def _cis(self, series_ring, base_ring):
        r"""
        The cycle index series for the species of subsets satisfies.

        .. MATH::

             Z_{\mathfrak{p}} = Z_{\mathcal{E}} \cdot Z_{\mathcal{E}}.

        EXAMPLES::

            sage: S = species.SubsetSpecies()
            sage: S.cycle_index_series()[0:5]                                           # needs sage.modules
            [p[],
             2*p[1],
             2*p[1, 1] + p[2],
             4/3*p[1, 1, 1] + 2*p[2, 1] + 2/3*p[3],
             2/3*p[1, 1, 1, 1] + 2*p[2, 1, 1] + 1/2*p[2, 2] + 4/3*p[3, 1] + 1/2*p[4]]
        """
        ciset = SetSpecies().cycle_index_series(base_ring)
        res = ciset**2
        if self.is_weighted():
            res *= self._weight
        return res


#Backward compatibility
SubsetSpecies_class = SubsetSpecies



# sage_setup: distribution = sagemath-combinat
# sage.doctest: needs sage.combinat sage.graphs sage.modules
r"""
Abstract classes for the rigged configuration bijections

This file contains two sets of classes, one for the bijection from KR tableaux to
rigged configurations and the other for the reverse bijection. We do this for
two reasons, one is because we can store a state in the bijection locally, so
we do not have to constantly pass it around between functions. The other is because
it makes the code easier to read in the \*_element.py files.

These classes are not meant to be used by the user and are only supposed to be
used internally to perform the bijections between
:class:`~sage.combinat.rigged_configurations.tensor_product_kr_tableaux.TensorProductOfKirillovReshetikhinTableaux`
and :class:`RiggedConfigurations`.

AUTHORS:

- Travis Scrimshaw (2011-04-15): Initial version
"""

# ****************************************************************************
#       Copyright (C) 2011, 2012 Travis Scrimshaw <tscrim@ucdavis.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#    This code is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    General Public License for more details.
#
#  The full text of the GPL is available at:
#
#                  https://www.gnu.org/licenses/
# ****************************************************************************

from copy import deepcopy
from sage.misc.abstract_method import abstract_method


class KRTToRCBijectionAbstract:
    """
    Root abstract class for the bijection from KR tableaux to rigged configurations.

    This class holds the state of the bijection and generates the next state.
    This class should never be created directly.
    """

    def __init__(self, tp_krt):
        """
        Initialize the bijection by obtaining the important information from
        the KR tableaux.

        INPUT:

        - ``parent`` -- the parent of tensor product of KR tableaux

        EXAMPLES::

            sage: KRT = crystals.TensorProductOfKirillovReshetikhinTableaux(['A', 4, 1], [[2,1]])
            sage: from sage.combinat.rigged_configurations.bij_type_A import KRTToRCBijectionTypeA
            sage: bijection = KRTToRCBijectionTypeA(KRT(pathlist=[[3,1]]))
            sage: TestSuite(bijection).run()
        """
        self.tp_krt = tp_krt
        self.n = tp_krt.parent().cartan_type().classical().rank()
        self.ret_rig_con = tp_krt.parent().rigged_configurations()(partition_list=[[]] * self.n)
        # We allow this to be mutable to make the bijection easier to program.
        # Upon completing the bijection, this will be set to immutable.
        # Do not call this, the object could be in a mutable state and ultimately
        #   be placed in an unstable state.
        # The user will (and should) never know about this temporary mutable state.
        self.ret_rig_con._set_mutable()
        self.cur_dims = []
        self.cur_path = []
        # self.L = {}

    def __eq__(self, rhs):
        r"""
        Check equality.

        This is only here for pickling check. This is a temporary placeholder
        class, and as such, should never be compared.

        TESTS::

            sage: KRT = crystals.TensorProductOfKirillovReshetikhinTableaux(['A', 4, 1], [[2,1]])
            sage: from sage.combinat.rigged_configurations.bij_type_A import KRTToRCBijectionTypeA
            sage: bijection = KRTToRCBijectionTypeA(KRT(pathlist=[[5,3]]))
            sage: bijection2 = KRTToRCBijectionTypeA(KRT(pathlist=[[5,3]]))
            sage: bijection == bijection2
            True
        """
        return isinstance(rhs, KRTToRCBijectionAbstract)

    def run(self, verbose=False):
        """
        Run the bijection from a tensor product of KR tableaux to a rigged
        configuration.

        INPUT:

        - ``tp_krt`` -- a tensor product of KR tableaux

        - ``verbose`` -- (default: ``False``) display each step in the
          bijection

        EXAMPLES::

            sage: KRT = crystals.TensorProductOfKirillovReshetikhinTableaux(['A', 4, 1], [[2, 1]])
            sage: from sage.combinat.rigged_configurations.bij_type_A import KRTToRCBijectionTypeA
            sage: KRTToRCBijectionTypeA(KRT(pathlist=[[5,2]])).run()
            <BLANKLINE>
            -1[ ]-1
            <BLANKLINE>
            1[ ]1
            <BLANKLINE>
            0[ ]0
            <BLANKLINE>
            -1[ ]-1
            <BLANKLINE>
        """
        if verbose:
            from sage.combinat.rigged_configurations.tensor_product_kr_tableaux_element \
                import TensorProductOfKirillovReshetikhinTableauxElement

        for cur_crystal in reversed(self.tp_krt):
            target = cur_crystal.parent()._r
            # Iterate through the columns
            for col_number, cur_column in enumerate(reversed(cur_crystal.to_array(False))):
                self.cur_path.insert(0, [])  # Prepend an empty list

                self.cur_dims.insert(0, [0, 1])

                for letter in reversed(cur_column):
                    self.cur_dims[0][0] = self._next_index(self.cur_dims[0][0], target)
                    val = letter.value  # Convert from a CrystalOfLetter to an Integer

                    if verbose:
                        print("====================")
                        print(repr(TensorProductOfKirillovReshetikhinTableauxElement(self.tp_krt.parent(), self.cur_path)))
                        print("--------------------")
                        print(repr(self.ret_rig_con))
                        print("--------------------\n")

                    # Build the next state
                    self.cur_path[0].insert(0, [letter])  # Prepend the value
                    self.next_state(val)

                # If we've split off a column, we need to merge the current column
                #   to the current crystal tableau
                if col_number > 0:
                    if verbose:
                        print("====================")
                        print(repr(TensorProductOfKirillovReshetikhinTableauxElement(self.tp_krt.parent(), self.cur_path)))
                        print("--------------------")
                        print(repr(self.ret_rig_con))
                        print("--------------------\n")
                        print("Applying column merge")

                    for i, letter_singleton in enumerate(self.cur_path[0]):
                        self.cur_path[1][i].insert(0, letter_singleton[0])
                    self.cur_dims[1][1] += 1
                    self.cur_path.pop(0)
                    self.cur_dims.pop(0)

                    # And perform the inverse column splitting map on the RC
                    for a in range(self.n):
                        self._update_vacancy_nums(a)

        self.ret_rig_con.set_immutable()  # Return it to immutable
        return self.ret_rig_con

    @abstract_method
    def next_state(self, val):
        r"""
        Build the next state in the bijection.

        INPUT:

        - ``val`` -- the value we are adding

        TESTS::

            sage: KRT = crystals.TensorProductOfKirillovReshetikhinTableaux(['A', 4, 1], [[2,1]])
            sage: from sage.combinat.rigged_configurations.bij_type_A import KRTToRCBijectionTypeA
            sage: bijection = KRTToRCBijectionTypeA(KRT(pathlist=[[5,3]]))
            sage: bijection.cur_path.insert(0, [])
            sage: bijection.cur_dims.insert(0, [0, 1])
            sage: bijection.cur_path[0].insert(0, [3])
            sage: bijection.next_state(3)
            sage: bijection.ret_rig_con
            <BLANKLINE>
            -1[ ]-1
            <BLANKLINE>
            -1[ ]-1
            <BLANKLINE>
            (/)
            <BLANKLINE>
            (/)
            <BLANKLINE>
        """

    def _update_vacancy_nums(self, a):
        r"""
        Update the vacancy numbers of a rigged partition.

        Helper function to (batch) update the vacancy numbers of the rigged
        partition at position `a` in the rigged configuration stored by this
        bijection.

        INPUT:

        - ``a`` -- the index of the partition to update

        TESTS::

            sage: KRT = crystals.TensorProductOfKirillovReshetikhinTableaux(['A', 4, 1], [[2,1]])
            sage: from sage.combinat.rigged_configurations.bij_abstract_class import KRTToRCBijectionAbstract
            sage: bijection = KRTToRCBijectionAbstract(KRT(pathlist=[[3,2]]))
            sage: bijection._update_vacancy_nums(2)
        """
        # Check to make sure we have a valid index (currently removed)
        # If the current tableau is empty, there is nothing to do
        if not self.ret_rig_con[a]:  # Check to see if we have vacancy numbers
            return

        # Setup the first block
        block_len = self.ret_rig_con[a][0]
        nu = self.ret_rig_con.nu()
        vac_num = self.ret_rig_con.parent()._calc_vacancy_number(nu, a, nu[a][0],
                                                                 dims=self.cur_dims)

        for i, row_len in enumerate(self.ret_rig_con[a]):
            # If we've gone to a different sized block, then update the
            #   values which change when moving to a new block size
            if block_len != row_len:
                vac_num = self.ret_rig_con.parent()._calc_vacancy_number(nu, a, row_len,
                                                                         dims=self.cur_dims)
                block_len = row_len
            self.ret_rig_con[a].vacancy_numbers[i] = vac_num

    def _update_partition_values(self, a):
        r"""
        Update the partition values of a rigged partition.

        Helper function to update the partition values of a given rigged
        partition row. This will go through all of our partition values and set
        them to our vacancy number if the corresponding row has been changed
        (indicated by being set to ``None``).

        INPUT:

        - ``a`` -- the index of the partition to update

        TESTS::

            sage: KRT = crystals.TensorProductOfKirillovReshetikhinTableaux(['A', 4, 1], [[2,1]])
            sage: from sage.combinat.rigged_configurations.bij_abstract_class import KRTToRCBijectionAbstract
            sage: bijection = KRTToRCBijectionAbstract(KRT(pathlist=[[5,2]]))
            sage: bijection._update_partition_values(2)
        """
        rigged_partition = self.ret_rig_con[a]
        for index, value in enumerate(rigged_partition.rigging):
            if value is None:
                rigged_partition.rigging[index] = rigged_partition.vacancy_numbers[index]
                if index > 0 and rigged_partition[index - 1] == rigged_partition[index] \
                  and rigged_partition.rigging[index - 1] < rigged_partition.rigging[index]:
                    # If we need to reorder
                    pos = 0
                    width = rigged_partition[index]
                    val = rigged_partition.rigging[index]
                    for i in reversed(range(index - 1)):
                        if rigged_partition[i] > width or rigged_partition.rigging[i] >= val:
                            pos = i + 1
                            break

                    rigged_partition.rigging.pop(index)
                    rigged_partition.rigging.insert(pos, val)

    def _next_index(self, r, target):
        """
        Return the next index after ``r`` when performing a step
        in the bijection going towards ``target``.

        TESTS::

            sage: KRT = crystals.TensorProductOfKirillovReshetikhinTableaux(['A', 4, 1], [[2,1]])
            sage: from sage.combinat.rigged_configurations.bij_abstract_class import KRTToRCBijectionAbstract
            sage: bijection = KRTToRCBijectionAbstract(KRT(pathlist=[[5,2]]))
            sage: bijection._next_index(1, 2)
            2
        """
        return r + 1


class RCToKRTBijectionAbstract:
    """
    Root abstract class for the bijection from rigged configurations to
    tensor product of Kirillov-Reshetikhin tableaux.

    This class holds the state of the bijection and generates the next state.
    This class should never be created directly.
    """

    def __init__(self, RC_element):
        """
        Initialize the bijection helper.

        INPUT:

        - ``RC_element`` -- the rigged configuration

        EXAMPLES::

            sage: RC = RiggedConfigurations(['A', 4, 1], [[2, 1]])
            sage: from sage.combinat.rigged_configurations.bij_abstract_class import RCToKRTBijectionAbstract
            sage: bijection = RCToKRTBijectionAbstract(RC(partition_list=[[1],[1],[1],[1]]))
            sage: TestSuite(bijection).run()
        """
        # Make a mutable clone of the rigged configuration for the bijection
        # This will be deleted when the bijection is completed
        self.rigged_con = RC_element.__copy__()
        self.n = RC_element.parent().cartan_type().classical().rank()
        self.KRT = RC_element.parent().tensor_product_of_kirillov_reshetikhin_tableaux()

        # Make a (deep) copy of the dimensions for the bijection
        self.cur_dims = [list(x[:]) for x in self.rigged_con.parent().dims]

        # Note that this implementation of the bijection is destructive to cur_partitions,
        #   therefore we will make a (deep) copy of the partitions.
        # TODO: Convert from cur_partitions to rigged_con
        self.cur_partitions = deepcopy(list(self.rigged_con)[:])

        # This is a dummy edge to start the process
        cp = RC_element.__copy__()
        cp.set_immutable()
        self._graph = [[[], (cp, 0)]]

    def __eq__(self, rhs):
        r"""
        Check equality.

        This is only here for pickling check. This is a temporary placeholder
        class, and as such, should never be compared.

        TESTS::

            sage: RC = RiggedConfigurations(['A', 4, 1], [[2, 1]])
            sage: from sage.combinat.rigged_configurations.bij_type_A import RCToKRTBijectionTypeA
            sage: bijection = RCToKRTBijectionTypeA(RC(partition_list=[[1],[1],[1],[1]]))
            sage: bijection2 = RCToKRTBijectionTypeA(RC(partition_list=[[1],[1],[1],[1]]))
            sage: bijection == bijection2
            True
        """
        return isinstance(rhs, RCToKRTBijectionAbstract)

    def run(self, verbose=False, build_graph=False):
        """
        Run the bijection from rigged configurations to tensor product of KR
        tableaux.

        INPUT:

        - ``verbose`` -- boolean (default: ``False``); display each step in the
          bijection
        - ``build_graph`` -- boolean (default: ``False``); build the graph of each
          step of the bijection

        EXAMPLES::

            sage: RC = RiggedConfigurations(['A', 4, 1], [[2, 1]])
            sage: x = RC(partition_list=[[1],[1],[1],[1]])
            sage: from sage.combinat.rigged_configurations.bij_type_A import RCToKRTBijectionTypeA
            sage: RCToKRTBijectionTypeA(x).run()
            [[2], [5]]
            sage: bij = RCToKRTBijectionTypeA(x)
            sage: bij.run(build_graph=True)
            [[2], [5]]
            sage: bij._graph
            Digraph on 3 vertices
        """
        from sage.combinat.crystals.letters import CrystalOfLetters
        letters = CrystalOfLetters(self.rigged_con.parent()._cartan_type.classical())

        # This is technically bad, but because the first thing we do is append
        #   an empty list to ret_crystal_path, we correct this. We do it this
        #   way so that we do not have to remove an empty list after the
        #   bijection has been performed.
        ret_crystal_path = []

        for dim in self.rigged_con.parent().dims:
            ret_crystal_path.append([])

            # Iterate over each column
            for dummy_var in range(dim[1]):
                # Split off a new column if necessary
                if self.cur_dims[0][1] > 1:
                    if verbose:
                        print("====================")
                        print(repr(self.rigged_con.parent()(*self.cur_partitions, use_vacancy_numbers=True)))
                        print("--------------------")
                        print(ret_crystal_path)
                        print("--------------------\n")
                        print("Applying column split")

                    self.cur_dims[0][1] -= 1
                    self.cur_dims.insert(0, [dim[0], 1])

                    # Perform the corresponding splitting map on rigged configurations
                    # All it does is update the vacancy numbers on the RC side
                    for a in range(self.n):
                        self._update_vacancy_numbers(a)

                    if build_graph:
                        y = self.rigged_con.parent()(*[x._clone() for x in self.cur_partitions], use_vacancy_numbers=True)
                        self._graph.append([self._graph[-1][1], (y, len(self._graph)), 'ls'])

                while self.cur_dims[0][0]:  # > 0:
                    if verbose:
                        print("====================")
                        print(repr(self.rigged_con.parent()(*self.cur_partitions, use_vacancy_numbers=True)))
                        print("--------------------")
                        print(ret_crystal_path)
                        print("--------------------\n")

                    ht = self.cur_dims[0][0]
                    self.cur_dims[0][0] = self._next_index(ht)
                    b = self.next_state(ht)

                    # Make sure we have a crystal letter
                    ret_crystal_path[-1].append(letters(b))  # Append the rank

                    if build_graph:
                        y = self.rigged_con.parent()(*[x._clone() for x in self.cur_partitions], use_vacancy_numbers=True)
                        self._graph.append([self._graph[-1][1], (y, len(self._graph)), letters(b)])

                self.cur_dims.pop(0)  # Pop off the leading column

        if build_graph:
            self._graph.pop(0)  # Remove the dummy at the start
            from sage.graphs.digraph import DiGraph
            from sage.graphs.dot2tex_utils import have_dot2tex
            self._graph = DiGraph(self._graph, format='list_of_edges')
            if have_dot2tex():
                self._graph.set_latex_options(format='dot2tex', edge_labels=True)
        return self.KRT(pathlist=ret_crystal_path)

    @abstract_method
    def next_state(self, height):
        """
        Build the next state in the bijection.

        TESTS::

            sage: RC = RiggedConfigurations(['A', 4, 1], [[2, 1]])
            sage: from sage.combinat.rigged_configurations.bij_type_A import RCToKRTBijectionTypeA
            sage: bijection = RCToKRTBijectionTypeA(RC(partition_list=[[1],[1],[1],[1]]))
            sage: bijection.next_state(1)
            5
            sage: bijection.cur_partitions
            [(/)
             , (/)
             , (/)
             , (/)
             ]
        """

    def _update_vacancy_numbers(self, a):
        r"""
        Update the vacancy numbers during the bijection.

        INPUT:

        - ``a`` -- the index of the partition to update

        TESTS::

            sage: RC = RiggedConfigurations(['A', 4, 1], [[2, 1]])
            sage: from sage.combinat.rigged_configurations.bij_abstract_class import RCToKRTBijectionAbstract
            sage: bijection = RCToKRTBijectionAbstract(RC(partition_list=[[1],[1],[1],[1]]))
            sage: bijection._update_vacancy_numbers(2)
        """

        # Nothing to do if there the rigged partition is empty
        if not self.cur_partitions[a]:
            return

        partition = self.cur_partitions[a]

        # Setup the first block
        block_len = partition[0]
        vac_num = self.rigged_con.parent()._calc_vacancy_number(self.cur_partitions,
                                                                a, partition[0],
                                                                dims=self.cur_dims)

        for i, row_len in enumerate(self.cur_partitions[a]):
            # If we've gone to a different sized block, then update the
            #   values which change when moving to a new block size
            if block_len != row_len:
                vac_num = self.rigged_con.parent()._calc_vacancy_number(self.cur_partitions,
                                                                        a, row_len,
                                                                        dims=self.cur_dims)
                block_len = row_len

            partition.vacancy_numbers[i] = vac_num

    def _find_singular_string(self, partition, last_size):
        r"""
        Return the index of the singular string or ``None`` if not found.

        Helper method to find a singular string at least as long as
        ``last_size``.

        INPUT:

        - ``partition`` -- the partition to look in

        - ``last_size`` -- the last size found

        TESTS::

            sage: RC = RiggedConfigurations(['A', 4, 1], [[2, 1]])
            sage: from sage.combinat.rigged_configurations.bij_abstract_class import RCToKRTBijectionAbstract
            sage: bijection = RCToKRTBijectionAbstract(RC(partition_list=[[1],[1],[1],[1]]))
            sage: bijection._find_singular_string(bijection.cur_partitions[2], 2)
            sage: bijection._find_singular_string(bijection.cur_partitions[2], 0)
            0
        """
        for i in reversed(range(len(partition))):
            if (partition[i] >= last_size
                    and partition.vacancy_numbers[i] == partition.rigging[i]):
                return i

    def _next_index(self, r):
        """
        Return the next index after ``r`` when performing a step
        in the bijection.

        TESTS::

            sage: RC = RiggedConfigurations(['A', 4, 1], [[2, 1]])
            sage: from sage.combinat.rigged_configurations.bij_abstract_class import RCToKRTBijectionAbstract
            sage: bijection = RCToKRTBijectionAbstract(RC(partition_list=[[1],[1],[1],[1]]))
            sage: bijection._next_index(2)
            1
        """
        return r - 1



# sage_setup: distribution = sagemath-combinat
# sage.doctest: needs sage.combinat sage.graphs sage.modules
r"""
Bijection classes for type `A_n^{(1)}`

Part of the (internal) classes which run the bijection between rigged
configurations and tensor products of Kirillov-Reshetikhin tableaux of
type `A_n^{(1)}`.

AUTHORS:

- Travis Scrimshaw (2011-04-15): Initial version

TESTS::

    sage: KRT = crystals.TensorProductOfKirillovReshetikhinTableaux(['A', 4, 1], [[2,1]])
    sage: from sage.combinat.rigged_configurations.bij_type_A import KRTToRCBijectionTypeA
    sage: bijection = KRTToRCBijectionTypeA(KRT(pathlist=[[5,2]]))
    sage: TestSuite(bijection).run()
    sage: RC = RiggedConfigurations(['A', 4, 1], [[2, 1]])
    sage: from sage.combinat.rigged_configurations.bij_type_A import RCToKRTBijectionTypeA
    sage: bijection = RCToKRTBijectionTypeA(RC(partition_list=[[],[],[],[]]))
    sage: TestSuite(bijection).run()
"""

# ****************************************************************************
#       Copyright (C) 2011, 2012 Travis Scrimshaw <tscrim@ucdavis.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#    This code is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    General Public License for more details.
#
#  The full text of the GPL is available at:
#
#                  https://www.gnu.org/licenses/
# ****************************************************************************

from sage.combinat.rigged_configurations.bij_abstract_class import KRTToRCBijectionAbstract
from sage.combinat.rigged_configurations.bij_abstract_class import RCToKRTBijectionAbstract


class KRTToRCBijectionTypeA(KRTToRCBijectionAbstract):
    r"""
    Specific implementation of the bijection from KR tableaux to rigged
    configurations for type `A_n^{(1)}`.
    """

    def next_state(self, val):
        r"""
        Build the next state for type `A_n^{(1)}`.

        EXAMPLES::

            sage: KRT = crystals.TensorProductOfKirillovReshetikhinTableaux(['A', 4, 1], [[2,1]])
            sage: from sage.combinat.rigged_configurations.bij_type_A import KRTToRCBijectionTypeA
            sage: bijection = KRTToRCBijectionTypeA(KRT(pathlist=[[4,3]]))
            sage: bijection.cur_path.insert(0, [])
            sage: bijection.cur_dims.insert(0, [0, 1])
            sage: bijection.cur_path[0].insert(0, [3])
            sage: bijection.next_state(3)
        """
        tableau_height = len(self.cur_path[0]) - 1
        n = self.n

        # Note first we subtract off for the n = max value (in the path) - 1,
        #   then we remove 1 to match the indices between math and programming.
        if val - 1 > tableau_height:
            # Always add a cell to the first singular value in the first
            #   tableau we are updating.
            if len(self.ret_rig_con[val - 2]) > 0:
                max_width = self.ret_rig_con[val - 2][0]
            else:
                max_width = 1

            # Insert a cell into the rightmost rigged partition
            max_width = self.ret_rig_con[val - 2].insert_cell(max_width)

            # Move to the left and update values as we have finished modifying
            #   everything which affects its vacancy/partition values
            for a in reversed(range(tableau_height, val - 2)):
                max_width = self.ret_rig_con[a].insert_cell(max_width)
                self._update_vacancy_nums(a + 1)
                self._update_partition_values(a + 1)

            # Update the final rigged tableau
            # Note if tabelauHeight = n+1, then we must have val = n+1 (in order
            #   to be column strict increasing), but then tableau_height is never
            #   greater than val, so we don't enter into this statement.
            self._update_vacancy_nums(tableau_height)
            self._update_partition_values(tableau_height)

            if val - 1 < n:
                self._update_vacancy_nums(val - 1)

            if tableau_height > 0:
                self._update_vacancy_nums(tableau_height - 1)
        elif tableau_height - 1 < n:
            # Otherwise we just need to update the vacancy numbers that are affected
            if tableau_height < n:
                self._update_vacancy_nums(tableau_height)

            if tableau_height > 0:
                self._update_vacancy_nums(tableau_height - 1)


class RCToKRTBijectionTypeA(RCToKRTBijectionAbstract):
    r"""
    Specific implementation of the bijection from rigged configurations to
    tensor products of KR tableaux for type `A_n^{(1)}`.
    """

    def next_state(self, height):
        r"""
        Build the next state for type `A_n^{(1)}`.

        EXAMPLES::

            sage: RC = RiggedConfigurations(['A', 4, 1], [[2, 1]])
            sage: from sage.combinat.rigged_configurations.bij_type_A import RCToKRTBijectionTypeA
            sage: bijection = RCToKRTBijectionTypeA(RC(partition_list=[[1],[1],[1],[1]]))
            sage: bijection.next_state(1)
            5
        """
        height -= 1  # indexing
        n = self.n
        ell = [None] * n
        b = None

        # Calculate the rank and ell values
        last_size = 0
        a = height
        for partition in self.cur_partitions[height:]:
            ell[a] = self._find_singular_string(partition, last_size)

            if ell[a] is None:
                b = a + 1
                break
            else:
                last_size = partition[ell[a]]
            a += 1

        if b is None:
            b = n + 1

        # Determine the new rigged configuration by removing a box from the selected
        #   string and then making the new string singular
        row_num = self.cur_partitions[0].remove_cell(ell[0])
        for a in range(1, n):
            row_num_next = self.cur_partitions[a].remove_cell(ell[a])

            self._update_vacancy_numbers(a - 1)
            if row_num is not None:
                self.cur_partitions[a - 1].rigging[row_num] = self.cur_partitions[a - 1].vacancy_numbers[row_num]
            row_num = row_num_next

        self._update_vacancy_numbers(n - 1)
        if row_num is not None:
            self.cur_partitions[n - 1].rigging[row_num] = self.cur_partitions[n - 1].vacancy_numbers[row_num]

        return b



# sage_setup: distribution = sagemath-combinat
# sage.doctest: needs sage.combinat sage.modules
r"""
Sudoku Puzzles

This module provides algorithms to solve Sudoku puzzles, plus tools
for inputting, converting and displaying various ways of writing a
puzzle or its solution(s).  Primarily this is accomplished with the
:class:`sage.games.sudoku.Sudoku` class, though the legacy top-level
:func:`sage.games.sudoku.sudoku` function is also available.

AUTHORS:

- Tom Boothby (2008/05/02): Exact Cover, Dancing Links algorithm
- Robert Beezer (2009/05/29): Backtracking algorithm, Sudoku class
"""
######################################################################
#       Copyright (C) 2009, Robert A. Beezer <beezer@ups.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#  The full text of the GPL is available at:
#                  https://www.gnu.org/licenses/
######################################################################

from sage.structure.sage_object import SageObject


def sudoku(m):
    r"""
    Solve Sudoku puzzles described by matrices.

    INPUT:

    - ``m`` -- a square Sage matrix over `\ZZ`, where zeros are blank entries

    OUTPUT:

    A Sage matrix over `\ZZ` containing the first solution found,
    otherwise ``None``.

    This function matches the behavior of the prior Sudoku solver
    and is included only to replicate that behavior.  It could be
    safely deprecated, since all of its functionality is included in the :class:`~sage.games.sudoku.Sudoku` class.

    EXAMPLES:

    An example that was used in previous doctests. ::

        sage: A = matrix(ZZ,9,[5,0,0, 0,8,0, 0,4,9, 0,0,0, 5,0,0, 0,3,0, 0,6,7, 3,0,0, 0,0,1, 1,5,0, 0,0,0, 0,0,0,  0,0,0, 2,0,8, 0,0,0, 0,0,0, 0,0,0, 0,1,8, 7,0,0, 0,0,4, 1,5,0, 0,3,0, 0,0,2, 0,0,0, 4,9,0, 0,5,0, 0,0,3])
        sage: A
        [5 0 0 0 8 0 0 4 9]
        [0 0 0 5 0 0 0 3 0]
        [0 6 7 3 0 0 0 0 1]
        [1 5 0 0 0 0 0 0 0]
        [0 0 0 2 0 8 0 0 0]
        [0 0 0 0 0 0 0 1 8]
        [7 0 0 0 0 4 1 5 0]
        [0 3 0 0 0 2 0 0 0]
        [4 9 0 0 5 0 0 0 3]
        sage: sudoku(A)
        [5 1 3 6 8 7 2 4 9]
        [8 4 9 5 2 1 6 3 7]
        [2 6 7 3 4 9 5 8 1]
        [1 5 8 4 6 3 9 7 2]
        [9 7 4 2 1 8 3 6 5]
        [3 2 6 7 9 5 4 1 8]
        [7 8 2 9 3 4 1 5 6]
        [6 3 5 1 7 2 8 9 4]
        [4 9 1 8 5 6 7 2 3]

    Using inputs that are possible with the
    :class:`~sage.games.sudoku.Sudoku` class,
    other than a matrix, will cause an error. ::

        sage: sudoku('.4..32....14..3.')
        Traceback (most recent call last):
        ...
        ValueError: sudoku function expects puzzle to be a matrix, perhaps use the Sudoku class
    """
    from sage.structure.element import Matrix

    if not isinstance(m, Matrix):
        raise ValueError('sudoku function expects puzzle to be a matrix, perhaps use the Sudoku class')
    solution = next(Sudoku(m).solve(algorithm='dlx'))
    return (solution.to_matrix() if solution else None)


class Sudoku(SageObject):
    r"""
    An object representing a Sudoku puzzle.  Primarily the purpose is to
    solve the puzzle, but conversions between formats are also provided.

    INPUT:

    - ``puzzle`` -- the first argument can take one of three forms
        * list - a Python list with elements of the puzzle in row-major order,
          where a blank entry is a zero
        * matrix - a square Sage matrix over `\ZZ`
        * string - a string where each character is an entry of
          the puzzle. For two-digit entries, a = 10, b = 11, etc.
    - ``verify_input`` -- boolean (default: ``True``); use ``False`` if you know
      the input is valid

    EXAMPLES::

        sage: a = Sudoku('5...8..49...5...3..673....115..........2.8..........187....415..3...2...49..5...3')
        sage: print(a)
        +-----+-----+-----+
        |5    |  8  |  4 9|
        |     |5    |  3  |
        |  6 7|3    |    1|
        +-----+-----+-----+
        |1 5  |     |     |
        |     |2   8|     |
        |     |     |  1 8|
        +-----+-----+-----+
        |7    |    4|1 5  |
        |  3  |    2|     |
        |4 9  |  5  |    3|
        +-----+-----+-----+
        sage: print(next(a.solve()))
        +-----+-----+-----+
        |5 1 3|6 8 7|2 4 9|
        |8 4 9|5 2 1|6 3 7|
        |2 6 7|3 4 9|5 8 1|
        +-----+-----+-----+
        |1 5 8|4 6 3|9 7 2|
        |9 7 4|2 1 8|3 6 5|
        |3 2 6|7 9 5|4 1 8|
        +-----+-----+-----+
        |7 8 2|9 3 4|1 5 6|
        |6 3 5|1 7 2|8 9 4|
        |4 9 1|8 5 6|7 2 3|
        +-----+-----+-----+
    """
    def __init__(self, puzzle, verify_input=True):
        r"""
        Initialize a Sudoku puzzle, determine its size, sanity-check the inputs.

        TESTS::

            sage: d = Sudoku('1.......2.9.4...5...6...7...5.9.3.......7.......85..4.7.....6...3...9.8...2.....1')
            sage: d == loads(dumps(d))
            True

        A lame attempt to construct a puzzle from a single integer::

            sage: Sudoku(8)
            Traceback (most recent call last):
            ...
            ValueError: Sudoku puzzle must be specified as a matrix, list or string

        An attempt to construct a puzzle from a non-square matrix::

            sage: Sudoku(matrix(2,range(6)))
            Traceback (most recent call last):
            ...
            ValueError: Sudoku puzzle must be a square matrix

        An attempt to construct a puzzle from a string of an impossible length (9)::

            sage: Sudoku('.........')
            Traceback (most recent call last):
            ...
            ValueError: Sudoku puzzle dimension of 3 must be a perfect square

        An attempt to construct a `4\times 4` puzzle from a string with a bad entry (5)::

            sage: Sudoku('.1.2.......5....')
            Traceback (most recent call last):
            ...
            ValueError: Sudoku puzzle has an invalid entry
        """
        from math import sqrt
        from sage.structure.element import Matrix

        if isinstance(puzzle, list):
            puzzle_size = int(round(sqrt(len(puzzle))))
            self.puzzle = tuple(puzzle)
        elif isinstance(puzzle, Matrix):
            puzzle_size = puzzle.ncols()
            if verify_input and not puzzle.is_square():
                raise ValueError('Sudoku puzzle must be a square matrix')
            self.puzzle = tuple([int(x) for x in puzzle.list()])
        elif isinstance(puzzle, str):
            puzzle_size = int(round(sqrt(len(puzzle))))
            puzzle_numeric = []
            for char in puzzle:
                if char.isdigit():
                    puzzle_numeric.append(int(char))
                elif char == '.':
                    puzzle_numeric.append(0)
                else:
                    puzzle_numeric.append(ord(char.upper()) - ord('A')+10)
            self.puzzle = tuple(puzzle_numeric)
        else:
            raise ValueError('Sudoku puzzle must be specified as a matrix, list or string')
        self.n = int(sqrt(puzzle_size))
        if verify_input:
            if self.n**4 != len(self.puzzle):
                raise ValueError('Sudoku puzzle dimension of %s must be a perfect square' % puzzle_size)
            for x in self.puzzle:
                if (x < 0) or (x > self.n*self.n):
                    raise ValueError('Sudoku puzzle has an invalid entry')

    def __eq__(self, other):
        r"""
        Compare two Sudoku puzzles, based on the underlying
        representation of the puzzles as tuples.

        EXAMPLES::

            sage: a = Sudoku('.4..32....14..3.')
            sage: b = Sudoku('8..6..9.5.............2.31...7318.6.24.....73...........279.1..5...8..36..3......')
            sage: c = Sudoku('1..6..9.5.............2.31...7318.6.24.....73...........279.1..5...8..36..3......')
            sage: d = Sudoku('81.6..9.5.............2.31...7318.6.24.....73...........279.1..5...8..36..3......')

            sage: a == b
            False
            sage: b == b
            True
            sage: b == c
            False
            sage: b == d
            False
        """
        return self.puzzle == tuple(other.to_list())

    def __hash__(self):
        """
        Return the hash of ``self``.

        EXAMPLES::

            sage: a = Sudoku('.4..32....14..3.')
            sage: hash(a) == hash(a.puzzle)
            True
        """
        return hash(self.puzzle)

    def __ne__(self, other):
        """
        Check that ``self`` is not equal to ``other``.

        EXAMPLES::

            sage: a = Sudoku('.4..32....14..3.')
            sage: b = Sudoku('8..6..9.5.............2.31...7318.6.24.....73...........279.1..5...8..36..3......')
            sage: c = Sudoku('1..6..9.5.............2.31...7318.6.24.....73...........279.1..5...8..36..3......')
            sage: d = Sudoku('81.6..9.5.............2.31...7318.6.24.....73...........279.1..5...8..36..3......')

            sage: a != b
            True
            sage: b != b
            False
            sage: b != c
            True
            sage: b != d
            True
        """
        return not (self == other)

    def _repr_(self):
        r"""
        Return a concise description of a Sudoku puzzle using a string representation.

        See the docstring for :func:`to_ascii` for more information on the format.

        EXAMPLES::

            sage: s = Sudoku('.4..32....14..3.')
            sage: s._repr_()
            '+---+---+\n|  4|   |\n|3 2|   |\n+---+---+\n|   |1 4|\n|   |3  |\n+---+---+'
        """
        return self.to_ascii()

    def _latex_(self):
        r"""nodetex
        Return a `\LaTeX` representation of a Sudoku puzzle as an array environment.

        EXAMPLES::

            sage: s = Sudoku('.4..32....14..3.')
            sage: s._latex_()
            '\\begin{array}{|*{2}{*{2}{r}|}}\\hline\n &4& & \\\\\n3&2& & \\\\\\hline\n & &1&4\\\\\n & &3& \\\\\\hline\n\\end{array}'
        """
        return self.to_latex()

    def _matrix_(self, R=None):
        r"""
        Return the puzzle as a matrix to support Sage's
        :func:`~sage.matrix.constructor.matrix` constructor.

        The base ring will be `\ZZ` if ``None`` is provided,
        and it is an error to specify any other base ring.

        EXAMPLES::

            sage: k = Sudoku('.4..32....14..3.')
            sage: matrix(k) # indirect doctest
            [0 4 0 0]
            [3 2 0 0]
            [0 0 1 4]
            [0 0 3 0]
            sage: matrix(ZZ,k)
            [0 4 0 0]
            [3 2 0 0]
            [0 0 1 4]
            [0 0 3 0]
            sage: matrix(QQ,k)
            Traceback (most recent call last):
            ...
            ValueError: Sudoku puzzles only convert to matrices over Integer Ring, not Rational Field
        """
        from sage.rings.integer_ring import ZZ, IntegerRing_class
        if R and not isinstance(R, IntegerRing_class):
            raise ValueError('Sudoku puzzles only convert to matrices over %s, not %s' % (ZZ, R))
        return self.to_matrix()

    def to_string(self):
        r"""
        Construct a string representing a Sudoku puzzle.

        Blank entries are represented as periods, single
        digits are not converted and two digit entries are
        converted to lower-case letters where ``10 = a``,
        ``11 = b``, etc.  This scheme limits puzzles to
        at most 36 symbols.

        EXAMPLES::

            sage: b = matrix(ZZ, 9, 9, [ [0,0,0,0,1,0,9,0,0], [8,0,0,4,0,0,0,0,0], [2,0,0,0,0,0,0,0,0], [0,7,0,0,3,0,0,0,0], [0,0,0,0,0,0,2,0,4], [0,0,0,0,0,0,0,5,8], [0,6,0,0,0,0,1,3,0], [7,0,0,2,0,0,0,0,0], [0,0,0,8,0,0,0,0,0] ])
            sage: Sudoku(b).to_string()
            '....1.9..8..4.....2.........7..3..........2.4.......58.6....13.7..2........8.....'

        TESTS:

        This tests the conversion of alphabetic characters as well as the
        input and output of Sudoku puzzles as strings. ::

            sage: j = Sudoku([0, 0, 0, 0, 10, 0, 0, 6, 9, 0, 3, 0, 0, 0, 0, 1, 13, 0, 2, 0, 0, 0, 8, 0, 0, 0, 0, 14, 0, 4, 0, 0, 0, 0, 11, 0, 0, 0, 0, 5, 0, 0, 12, 0, 0, 0, 0, 0, 0, 0, 7, 0, 0, 0, 0, 0, 0, 0, 16, 0, 0, 0, 15, 0, 0, 0, 0, 1, 0, 14, 0, 0, 2, 0, 11, 0, 8, 0, 0, 3, 0, 0, 0, 0, 0, 0, 0, 4, 0, 13, 0, 0, 0, 0, 0, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 15, 0, 0, 7, 0, 16, 0, 0, 9, 0, 10, 0, 0, 12, 0, 0, 0, 5, 0, 0, 0, 0, 0, 8, 0, 0, 15, 0, 0, 0, 0, 0, 1, 0, 0, 14, 0, 7, 9, 0, 12, 0, 0, 0, 0, 11, 0, 0, 0, 0, 0, 2, 0, 0, 0, 6, 0, 0, 0, 0, 0, 16, 0, 7, 0, 0, 0, 0, 0, 0, 8, 4, 0, 0, 0, 0, 3, 0, 13, 0, 0, 10, 0, 5, 0, 0, 0, 0, 5, 0, 0, 0, 7, 0, 0, 14, 0, 0, 0, 12, 10, 0, 0, 0, 0, 0, 3, 0, 1, 0, 0, 0, 0, 0, 0, 0, 11, 0, 0, 0, 0, 0, 0, 15, 0, 0, 0, 0, 4, 0, 0, 0, 13, 0, 0, 14, 0, 0, 16, 0, 9, 2, 0, 6, 0, 0, 8, 0, 0, 0, 0])
            sage: st = j.to_string()
            sage: st
            '....a..69.3....1d.2...8....e.4....b....5..c.......7.......g...f....1.e..2.b.8..3.......4.d.....6.........f..7.g..9.a..c...5.....8..f.....1..e.79.c....b.....2...6.....g.7......84....3.d..a.5....5...7..e...ca.....3.1.......b......f....4...d..e..g.92.6..8....'
            sage: st == Sudoku(st).to_string()
            True

        A `49\times 49` puzzle with all entries equal to 40,
        which doesn't convert to a letter. ::

            sage: empty = [40]*2401
            sage: Sudoku(empty).to_string()
            Traceback (most recent call last):
            ...
            ValueError: Sudoku string representation is only valid for puzzles of size 36 or smaller
        """
        encoded = []
        for x in self.puzzle:
            if x == 0:
                encoded.append('.')
            elif 1 <= x <= 9:
                encoded.append(str(x))
            elif x <= 36:
                encoded.append(chr(x-10+ord('a')))
            else:
                raise ValueError('Sudoku string representation is only valid for puzzles of size 36 or smaller')
        return ''.join(encoded)

    def to_list(self):
        r"""
        Construct a list representing a Sudoku puzzle, in row-major order.

        EXAMPLES::

            sage: s = Sudoku('1.......2.9.4...5...6...7...5.9.3.......7.......85..4.7.....6...3...9.8...2.....1')
            sage: s.to_list()
            [1, 0, 0, 0, 0, 0, 0, 0, 2, 0, 9, 0, 4, 0, 0, 0, 5, 0, 0, 0, 6, 0, 0, 0, 7, 0, 0, 0, 5, 0, 9, 0, 3, 0, 0, 0, 0, 0, 0, 0, 7, 0, 0, 0, 0, 0, 0, 0, 8, 5, 0, 0, 4, 0, 7, 0, 0, 0, 0, 0, 6, 0, 0, 0, 3, 0, 0, 0, 9, 0, 8, 0, 0, 0, 2, 0, 0, 0, 0, 0, 1]

        TESTS:

        This tests the input and output of Sudoku puzzles as lists. ::

            sage: alist = [0, 4, 0, 0, 3, 2, 0, 0, 0, 0, 1, 4, 0, 0, 3, 0]
            sage: alist == Sudoku(alist).to_list()
            True
        """
        return list(self.puzzle)

    def to_matrix(self):
        r"""
        Construct a Sage matrix over `\ZZ` representing a Sudoku puzzle.

        EXAMPLES::

            sage: s = Sudoku('.4..32....14..3.')
            sage: s.to_matrix()
            [0 4 0 0]
            [3 2 0 0]
            [0 0 1 4]
            [0 0 3 0]

        TESTS:

        This tests the input and output of Sudoku puzzles as matrices over `\ZZ`. ::

            sage: g = matrix(ZZ, 9, 9, [ [1,0,0,0,0,7,0,9,0], [0,3,0,0,2,0,0,0,8], [0,0,9,6,0,0,5,0,0], [0,0,5,3,0,0,9,0,0], [0,1,0,0,8,0,0,0,2], [6,0,0,0,0,4,0,0,0], [3,0,0,0,0,0,0,1,0], [0,4,0,0,0,0,0,0,7], [0,0,7,0,0,0,3,0,0] ])
            sage: g == Sudoku(g).to_matrix()
            True
        """
        from sage.rings.integer_ring import ZZ
        from sage.matrix.constructor import matrix
        return matrix(ZZ, self.n*self.n, self.puzzle)

    def to_ascii(self):
        r"""
        Construct an ASCII-art version of a Sudoku puzzle.
        This is a modified version of the ASCII version of a subdivided matrix.

        EXAMPLES::

            sage: s = Sudoku('.4..32....14..3.')
            sage: print(s.to_ascii())
            +---+---+
            |  4|   |
            |3 2|   |
            +---+---+
            |   |1 4|
            |   |3  |
            +---+---+
            sage: s.to_ascii()
            '+---+---+\n|  4|   |\n|3 2|   |\n+---+---+\n|   |1 4|\n|   |3  |\n+---+---+'
        """
        from re import compile
        n = self.n
        nsquare = n*n
        m = self.to_matrix()
        m.subdivide(list(range(0,nsquare+1,n)), list(range(0,nsquare+1,n)))
        naked_zero = compile(r'([\|, ]+)0')
        blanked = naked_zero.sub(lambda x: x.group(1)+' ', m.str())
        brackets = compile(r'[\[,\]]')
        return brackets.sub('', blanked)

    def to_latex(self):
        r"""
        Create a string of `\LaTeX` code representing a Sudoku puzzle or solution.

        EXAMPLES::

            sage: s = Sudoku('.4..32....14..3.')
            sage: print(s.to_latex())
            \begin{array}{|*{2}{*{2}{r}|}}\hline
            &4& & \\
            3&2& & \\\hline
            & &1&4\\
            & &3& \\\hline
            \end{array}

        TESTS::

            sage: s = Sudoku('.4..32....14..3.')
            sage: s.to_latex()
            '\\begin{array}{|*{2}{*{2}{r}|}}\\hline\n &4& & \\\\\n3&2& & \\\\\\hline\n & &1&4\\\\\n & &3& \\\\\\hline\n\\end{array}'
        """
        n = self.n
        nsquare = n*n
        array = []
        array.append('\\begin{array}{|*{%s}{*{%s}{r}|}}\\hline\n' % (n, n))
        gen = iter(self.puzzle)
        for row in range(nsquare):
            for col in range(nsquare):
                entry = next(gen)
                array.append(str(entry) if entry else ' ')
                array.append('' if col == nsquare - 1 else '&')
            array.append('\\\\\n' if (row+1) % n else '\\\\\\hline\n')
        array.append('\\end{array}')
        return ''.join(array)

    def solve(self, algorithm='dlx'):
        r"""
        Return a generator object for the solutions of a Sudoku puzzle.

        INPUT:

        - ``algorithm`` -- string (default: ``'dlx'``); specify choice of
          solution algorithm. The two possible algorithms are ``'dlx'`` and
          ``'backtrack'``

        OUTPUT:

        A generator that provides all solutions, as objects of
        the :class:`~sage.games.sudoku.Sudoku` class.

        Calling ``next()`` on the returned generator just once will find
        a solution, presuming it exists, otherwise it will return a
        ``StopIteration`` exception.  The generator may be used for
        iteration or wrapping the generator with ``list()`` will return
        all of the solutions as a list.  Solutions are returned as
        new objects of the :class:`~sage.games.sudoku.Sudoku` class,
        so may be printed or converted using other methods in this class.

        Generally, the DLX algorithm is very fast and very consistent.
        The backtrack algorithm is very variable in its performance,
        on some occasions markedly faster than DLX but usually slower
        by a similar factor, with the potential to be orders of magnitude
        slower.  See the docstrings for the
        :meth:`~sage.games.sudoku.Sudoku.dlx` and
        :meth:`~sage.games.sudoku.Sudoku.backtrack_all`
        methods for further discussions and examples of performance.
        Note that the backtrack algorithm is limited to puzzles of
        size `16\times 16` or smaller.

        EXAMPLES:

        This puzzle has 5 solutions, but the first one returned by each algorithm are identical. ::

            sage: h = Sudoku('8..6..9.5.............2.31...7318.6.24.....73...........279.1..5...8..36..3......')
            sage: h
            +-----+-----+-----+
            |8    |6    |9   5|
            |     |     |     |
            |     |  2  |3 1  |
            +-----+-----+-----+
            |    7|3 1 8|  6  |
            |2 4  |     |  7 3|
            |     |     |     |
            +-----+-----+-----+
            |    2|7 9  |1    |
            |5    |  8  |  3 6|
            |    3|     |     |
            +-----+-----+-----+
            sage: next(h.solve(algorithm='backtrack'))
            +-----+-----+-----+
            |8 1 4|6 3 7|9 2 5|
            |3 2 5|1 4 9|6 8 7|
            |7 9 6|8 2 5|3 1 4|
            +-----+-----+-----+
            |9 5 7|3 1 8|4 6 2|
            |2 4 1|9 5 6|8 7 3|
            |6 3 8|2 7 4|5 9 1|
            +-----+-----+-----+
            |4 6 2|7 9 3|1 5 8|
            |5 7 9|4 8 1|2 3 6|
            |1 8 3|5 6 2|7 4 9|
            +-----+-----+-----+
            sage: next(h.solve(algorithm='dlx'))
            +-----+-----+-----+
            |8 1 4|6 3 7|9 2 5|
            |3 2 5|1 4 9|6 8 7|
            |7 9 6|8 2 5|3 1 4|
            +-----+-----+-----+
            |9 5 7|3 1 8|4 6 2|
            |2 4 1|9 5 6|8 7 3|
            |6 3 8|2 7 4|5 9 1|
            +-----+-----+-----+
            |4 6 2|7 9 3|1 5 8|
            |5 7 9|4 8 1|2 3 6|
            |1 8 3|5 6 2|7 4 9|
            +-----+-----+-----+

        Gordon Royle maintains a list of 48072 Sudoku puzzles that each has
        a unique solution and exactly 17 "hints" (initially filled boxes).
        At this writing (May 2009) there is no known 16-hint puzzle with
        exactly one solution.  [sudoku:royle]_  This puzzle is number 3000
        in his database.  We solve it twice. ::

            sage: b = Sudoku('8..6..9.5.............2.31...7318.6.24.....73...........279.1..5...8..36..3......')
            sage: next(b.solve(algorithm='dlx')) == next(b.solve(algorithm='backtrack'))
            True


        These are the first 10 puzzles in a list of "Top 95" puzzles,
        [sudoku:top95]_ which we use to show that the two available algorithms obtain
        the same solution for each. ::

            sage: top =['4.....8.5.3..........7......2.....6.....8.4......1.......6.3.7.5..2.....1.4......',
            ....:       '52...6.........7.13...........4..8..6......5...........418.........3..2...87.....',
            ....:       '6.....8.3.4.7.................5.4.7.3..2.....1.6.......2.....5.....8.6......1....',
            ....:       '48.3............71.2.......7.5....6....2..8.............1.76...3.....4......5....',
            ....:       '....14....3....2...7..........9...3.6.1.............8.2.....1.4....5.6.....7.8...',
            ....:       '......52..8.4......3...9...5.1...6..2..7........3.....6...1..........7.4.......3.',
            ....:       '6.2.5.........3.4..........43...8....1....2........7..5..27...........81...6.....',
            ....:       '.524.........7.1..............8.2...3.....6...9.5.....1.6.3...........897........',
            ....:       '6.2.5.........4.3..........43...8....1....2........7..5..27...........81...6.....',
            ....:       '.923.........8.1...........1.7.4...........658.........6.5.2...4.....7.....9.....']
            sage: p = [Sudoku(top[i]) for i in range(10)]
            sage: verify = [next(p[i].solve(algorithm='dlx')) == next(p[i].solve(algorithm='backtrack')) for i in range(10)]
            sage: verify == [True]*10
            True

        TESTS:

        A `25\times 25` puzzle that the backtrack algorithm is not equipped to handle.  Since ``solve`` returns a generator this test will not go boom until we ask for a solution with ``next``. ::

            sage: too_big = Sudoku([0]*625)
            sage: next(too_big.solve(algorithm='backtrack'))
            Traceback (most recent call last):
            ...
            ValueError: the Sudoku backtrack algorithm is limited to puzzles of size 16 or smaller

        An attempt to use a non-existent algorithm. ::

            sage: next(Sudoku([0]).solve(algorithm='bogus'))
            Traceback (most recent call last):
            ...
            NotImplementedError: bogus is not an algorithm for Sudoku puzzles
        """
        if algorithm == 'backtrack':
            if self.n > 4:
                raise ValueError('the Sudoku backtrack algorithm is limited to puzzles of size 16 or smaller')
            else:
                gen = self.backtrack()
        elif algorithm == 'dlx':
            gen = self.dlx()
        else:
            raise NotImplementedError('%s is not an algorithm for Sudoku puzzles' % algorithm)
        for soln in gen:
            yield Sudoku(soln, verify_input='False')

    def backtrack(self):
        r"""
        Return a generator which iterates through all solutions of a Sudoku puzzle.

        This function is intended to be called from the
        :func:`~sage.games.sudoku.Sudoku.solve` method
        when the ``algorithm='backtrack'`` option is specified.
        However it may be called directly as a method of an
        instance of a Sudoku puzzle.

        At this point, this method calls
        :func:`~sage.games.sudoku_backtrack.backtrack_all` which
        constructs *all* of the solutions as a list.  Then the
        present method just returns the items of the list one at
        a time.  Once Cython supports closures and a yield statement
        is supported, then the contents of ``backtrack_all()``
        may be subsumed into this method and the
        :mod:`sage.games.sudoku_backtrack` module can be removed.

        This routine can have wildly variable performance, with a
        factor of 4000 observed between the fastest and slowest
        `9\times 9` examples tested. Examples designed to perform
        poorly for naive backtracking, will do poorly
        (such as ``d`` below).  However, examples meant to be
        difficult for humans often do very well, with a factor
        of 5 improvement over the `DLX` algorithm.

        Without dynamically allocating arrays in the Cython version,
        we have limited this function to `16\times 16` puzzles.
        Algorithmic details are in the
        :mod:`sage.games.sudoku_backtrack` module.

        EXAMPLES:

        This example was reported to be very difficult for human solvers.
        This algorithm works very fast on it, at about half the time
        of the DLX solver. [sudoku:escargot]_ ::

            sage: g = Sudoku('1....7.9..3..2...8..96..5....53..9...1..8...26....4...3......1..4......7..7...3..')
            sage: print(g)
            +-----+-----+-----+
            |1    |    7|  9  |
            |  3  |  2  |    8|
            |    9|6    |5    |
            +-----+-----+-----+
            |    5|3    |9    |
            |  1  |  8  |    2|
            |6    |    4|     |
            +-----+-----+-----+
            |3    |     |  1  |
            |  4  |     |    7|
            |    7|     |3    |
            +-----+-----+-----+
            sage: print(next(g.solve(algorithm='backtrack')))
            +-----+-----+-----+
            |1 6 2|8 5 7|4 9 3|
            |5 3 4|1 2 9|6 7 8|
            |7 8 9|6 4 3|5 2 1|
            +-----+-----+-----+
            |4 7 5|3 1 2|9 8 6|
            |9 1 3|5 8 6|7 4 2|
            |6 2 8|7 9 4|1 3 5|
            +-----+-----+-----+
            |3 5 6|4 7 8|2 1 9|
            |2 4 1|9 3 5|8 6 7|
            |8 9 7|2 6 1|3 5 4|
            +-----+-----+-----+

        This example has no entries in the top row and a half,
        and the top row of the solution is ``987654321`` and
        therefore a backtracking approach is slow, taking about
        750 times as long as the DLX solver. [sudoku:wikipedia]_ ::

            sage: c = Sudoku('..............3.85..1.2.......5.7.....4...1...9.......5......73..2.1........4...9')
            sage: print(c)
            +-----+-----+-----+
            |     |     |     |
            |     |    3|  8 5|
            |    1|  2  |     |
            +-----+-----+-----+
            |     |5   7|     |
            |    4|     |1    |
            |  9  |     |     |
            +-----+-----+-----+
            |5    |     |  7 3|
            |    2|  1  |     |
            |     |  4  |    9|
            +-----+-----+-----+
            sage: print(next(c.solve(algorithm='backtrack')))
            +-----+-----+-----+
            |9 8 7|6 5 4|3 2 1|
            |2 4 6|1 7 3|9 8 5|
            |3 5 1|9 2 8|7 4 6|
            +-----+-----+-----+
            |1 2 8|5 3 7|6 9 4|
            |6 3 4|8 9 2|1 5 7|
            |7 9 5|4 6 1|8 3 2|
            +-----+-----+-----+
            |5 1 9|2 8 6|4 7 3|
            |4 7 2|3 1 9|5 6 8|
            |8 6 3|7 4 5|2 1 9|
            +-----+-----+-----+
        """
        from .sudoku_backtrack import backtrack_all
        solutions = backtrack_all(self.n, self.puzzle)
        yield from solutions

    def dlx(self, count_only=False):
        r"""
        Return a generator that iterates through all solutions of a Sudoku puzzle.

        INPUT:

        - ``count_only`` -- boolean (default: ``False``);
          if set to ``True`` the generator returned as output will
          simply generate ``None`` for each solution, so the
          calling routine can count these

        OUTPUT: a generator that iterates over all the solutions

        This function is intended to be called from the
        :func:`~sage.games.sudoku.Sudoku.solve` method
        with the ``algorithm='dlx'`` option. However it
        may be called directly as a method of an instance
        of a Sudoku puzzle if speed is important and you
        do not need automatic conversions on the output
        (or even just want to count solutions without looking
        at them).  In this case, inputting a puzzle as a list,
        with ``verify_input=False`` is the fastest way to
        create a puzzle.

        Or if only one solution is needed it can be obtained with
        one call to ``next()``, while the existence of a solution
        can be tested by catching the ``StopIteration`` exception
        with a ``try``.  Calling this particular method returns
        solutions as lists, in row-major order. It is up to you
        to work with this list for your own purposes. If you want
        fancier formatting tools, use the
        :func:`~sage.games.sudoku.Sudoku.solve` method, which
        returns a generator that creates
        :class:`sage.games.sudoku.Sudoku` objects.

        EXAMPLES:

        A `9\times 9` known to have one solution.  We get the one
        solution and then check to see if there are more or not. ::

            sage: e = Sudoku('4.....8.5.3..........7......2.....6.....8.4......1.......6.3.7.5..2.....1.4......')
            sage: print(next(e.dlx()))
            [4, 1, 7, 3, 6, 9, 8, 2, 5, 6, 3, 2, 1, 5, 8, 9, 4, 7, 9, 5, 8, 7, 2, 4, 3, 1, 6, 8, 2, 5, 4, 3, 7, 1, 6, 9, 7, 9, 1, 5, 8, 6, 4, 3, 2, 3, 4, 6, 9, 1, 2, 7, 5, 8, 2, 8, 9, 6, 4, 3, 5, 7, 1, 5, 7, 3, 2, 9, 1, 6, 8, 4, 1, 6, 4, 8, 7, 5, 2, 9, 3]
            sage: len(list(e.dlx()))
            1

        A `9\times 9` puzzle with multiple solutions.
        Once with actual solutions, once just to count. ::

            sage: h = Sudoku('8..6..9.5.............2.31...7318.6.24.....73...........279.1..5...8..36..3......')
            sage: len(list(h.dlx()))
            5
            sage: len(list(h.dlx(count_only=True)))
            5

        A larger puzzle, with multiple solutions, but we just get one. ::

            sage: j = Sudoku('....a..69.3....1d.2...8....e.4....b....5..c.......7.......g...f....1.e..2.b.8..3.......4.d.....6.........f..7.g..9.a..c...5.....8..f.....1..e.79.c....b.....2...6.....g.7......84....3.d..a.5....5...7..e...ca.....3.1.......b......f....4...d..e..g.92.6..8....')
            sage: print(next(j.dlx()))
            [5, 15, 16, 14, 10, 13, 7, 6, 9, 2, 3, 4, 11, 8, 12, 1, 13, 3, 2, 12, 11, 16, 8, 15, 1, 6, 7, 14, 10, 4, 9, 5, 1, 10, 11, 6, 9, 4, 3, 5, 15, 8, 12, 13, 16, 7, 14, 2, 9, 8, 7, 4, 12, 2, 1, 14, 10, 5, 16, 11, 6, 3, 15, 13, 12, 16, 4, 1, 13, 14, 9, 10, 2, 7, 11, 6, 8, 15, 5, 3, 3, 14, 5, 7, 16, 11, 15, 4, 12, 13, 8, 9, 1, 2, 10, 6, 2, 6, 13, 11, 1, 8, 5, 3, 4, 15, 14, 10, 7, 9, 16, 12, 15, 9, 8, 10, 2, 6, 12, 7, 3, 16, 5, 1, 4, 14, 13, 11, 8, 11, 3, 15, 5, 10, 4, 2, 13, 1, 6, 12, 14, 16, 7, 9, 16, 12, 14, 13, 7, 15, 11, 1, 8, 9, 4, 5, 2, 6, 3, 10, 6, 2, 10, 5, 14, 12, 16, 9, 7, 11, 15, 3, 13, 1, 4, 8, 4, 7, 1, 9, 8, 3, 6, 13, 16, 14, 10, 2, 5, 12, 11, 15, 11, 5, 9, 8, 6, 7, 13, 16, 14, 3, 1, 15, 12, 10, 2, 4, 7, 13, 15, 3, 4, 1, 10, 8, 5, 12, 2, 16, 9, 11, 6, 14, 10, 1, 6, 2, 15, 5, 14, 12, 11, 4, 9, 7, 3, 13, 8, 16, 14, 4, 12, 16, 3, 9, 2, 11, 6, 10, 13, 8, 15, 5, 1, 7]

        The puzzle ``h`` from above, but purposely made unsolvable with addition in second entry. ::

            sage: hbad = Sudoku('82.6..9.5.............2.31...7318.6.24.....73...........279.1..5...8..36..3......')
            sage: len(list(hbad.dlx()))
            0
            sage: next(hbad.dlx())
            Traceback (most recent call last):
            ...
            StopIteration

        A stupidly small puzzle to test the lower limits of arbitrary sized input. ::

            sage: s = Sudoku('.')
            sage: print(next(s.solve(algorithm='dlx')))
            +-+
            |1|
            +-+

        ALGORITHM:

        The ``DLXCPP`` solver finds solutions to the exact-cover
        problem with a "Dancing Links" backtracking algorithm.
        Given a `0-1` matrix, the solver finds a subset of the
        rows that sums to the all `1`'s vector.  The columns
        correspond to conditions, or constraints, that must be
        met by a solution, while the rows correspond to some
        collection of choices, or decisions.  A `1` in a row
        and column indicates that the choice corresponding to
        the row meets the condition corresponding to the column.

        So here, we code the notion of a Sudoku puzzle, and the
        hints already present, into such a `0-1` matrix.  Then the
        :class:`sage.combinat.matrices.dlxcpp.DLXCPP` solver makes
        the choices for the blank entries.
        """
        from sage.combinat.matrices.dlxcpp import DLXCPP

        n = self.n
        nsquare = n*n
        nfour = nsquare*nsquare

        # Boxes of the grid are numbered in row-major order
        # ``rcbox`` simply maps a row-column index pair to the box number it lives in
        rcbox = [ [i//n + n*(j//n) for i in range(nsquare)] for j in range(nsquare)]

        # Every entry in a Sudoku puzzle satisfies four constraints
        # Every location has a single entry, and each row, column and box has each symbol once
        # These arrays can be thought of as assigning ID numbers to these constraints,
        # and correspond to column numbers of the `0-1` matrix describing the exact cover
        rows = [[i+j for i in range(nsquare)] for j in range(0, nfour, nsquare)]
        cols = [[i+j for i in range(nsquare)] for j in range(nfour, 2*nfour, nsquare)]
        boxes = [[i+j for i in range(nsquare)] for j in range(2*nfour, 3*nfour, nsquare)]
        rowcol = [[i+j for i in range(nsquare)] for j in range(3*nfour, 4*nfour, nsquare)]

        def make_row(row, col, entry):
            r"""
            Construct a row of the `0-1` matrix describing
            the exact cover constraints for a Sudoku puzzle.

            If a (zero-based) ``entry`` is placed in location
            ``(row, col)`` of a Sudoku puzzle, then exactly four
            constraints are satisfied: the location has this entry,
            and the entry occurs in the row, column and box.
            This method looks up the constraint IDs for each of
            these four constraints, and returns a list of these four IDs.

            TESTS::

                sage: h = Sudoku('8..6..9.5.............2.31...7318.6.24.....73...........279.1..5...8..36..3......')
                sage: len(list(h.solve(algorithm='dlx')))  # indirect doctest
                5
            """
            box = rcbox[row][col]
            return [rows[row][entry], cols[col][entry], boxes[box][entry], rowcol[row][col]]

        # Construct the sparse `0-1` matrix for the exact cover formulation as the ``ones`` array
        # ``rowinfo`` remembers the location and entry that led to the row being added to the matrix
        rowinfo = []
        ones = []
        gen = (entry for entry in self.puzzle)
        for row in range(nsquare):
            for col in range(nsquare):
                puzz = next(gen)
                # All (zero-based) entries are possible, or only one is possible
                entries = ([puzz-1] if puzz else range(nsquare))
                for entry in entries:
                    ones.append(make_row(row, col, entry))
                    rowinfo.append((row, col, entry))

        # ``DLXCPP`` will solve the exact cover problem for the ``ones`` matrix
        # ``cover`` will contain a subset of the row indices so that the sum of the rows is the all `1`'s vector
        # These rows will represent the original hints, plus a single entry in every other location,
        # consistent with the requirements imposed on a solution to a Sudoku puzzle
        for cover in DLXCPP(ones):
            if not count_only:
                solution = [0] * nfour
                for r in cover:
                    row, col, entry = rowinfo[r]
                    solution[row * nsquare + col] = entry + 1
                yield solution
            else:
                yield None



# sage_setup: distribution = sagemath-combinat
r"""
Sidon sets and their generalizations, Sidon `g`-sets

AUTHORS:

- Martin Raum (07-25-2011)
"""
# ****************************************************************************
#                 Copyright (C) 2011 Martin Raum
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  https://www.gnu.org/licenses/
# ****************************************************************************
from sage.sets.set import Set
from sage.misc.cachefunc import cached_function
from sage.rings.integer import Integer


def sidon_sets(N, g=1):
    r"""
    Return the set of all Sidon-`g` sets that have elements less than or equal
    to `N`.

    A Sidon-`g` set is a set of positive integers `A \subset [1, N]` such
    that any integer `M` can be obtain at most `g` times as sums of unordered pairs of
    elements of `A` (the two elements are not necessary distinct):

    .. MATH::

        \#\{ (a_i, a_j) | a_i, a_j \in A, a_i + a_j = M,a_i \leq a_j \} \leq g

    INPUT:

    - ``N`` -- positive integer
    - ``g`` -- positive integer (default: `1`)

    OUTPUT: a Sage set with categories whose element are also set of integers

    EXAMPLES::

        sage: S = sidon_sets(3, 2)
        sage: sorted(S, key=str)
        [{1, 2, 3}, {1, 2}, {1, 3}, {1}, {2, 3}, {2}, {3}, {}]
        sage: S.cardinality()
        8
        sage: S.category()
        Category of finite enumerated sets
        sage: sid = S.an_element()
        sage: sid
        {2}
        sage: sid.category()
        Category of finite enumerated sets

    TESTS::

        sage: S = sidon_sets(10)
        sage: TestSuite(S).run()
        sage: Set([1,2,4,8,13]) in sidon_sets(13)
        True

    The following piece of code computes the first values of the Sloane sequence
    entitled 'Length of shortest (or optimal) Golomb ruler with n marks' with a
    very dumb algorithm. (sequence identifier A003022)::

        sage: n = 1
        sage: L = []
        sage: for i in range(1,19):
        ....:     nb = max([S.cardinality() for S in sidon_sets(i)])
        ....:     if nb > n:
        ....:         L.append(i-1)
        ....:         n = nb
        sage: L
        [1, 3, 6, 11, 17]

    The following tests check that some generalized Sidon sets satisfy the right
    conditions, using a dumb but exhaustive algorithm::

        sage: from itertools import groupby
        sage: all(all(l <= 3 for l in map(lambda s: len(list(s[1])), groupby(sorted(a + ap for a in sid for ap in sid if a >= ap), lambda s: s))) for sid in sidon_sets(10, 3))
        True
        sage: all(all(l <= 5 for l in map(lambda s: len(list(s[1])), groupby(sorted(a + ap for a in sid for ap in sid if a >= ap), lambda s: s))) for sid in sidon_sets(10, 5))
        True

    Checking of arguments::

        sage: sidon_sets(1,1)
        {{}, {1}}
        sage: sidon_sets(-1,3)
        Traceback (most recent call last):
        ...
        ValueError: N must be a positive integer
        sage: sidon_sets(1, -3)
        Traceback (most recent call last):
        ...
        ValueError: g must be a positive integer
    """
    if not isinstance(N, (int, Integer)) or N < 1:
        raise ValueError( "N must be a positive integer" )
    elif not isinstance(g, (int, Integer)) or g < 1:
        raise ValueError( "g must be a positive integer" )
    return sidon_sets_rec(N, g=g)


# This recursive and cached slave function is mainly here because
# caching the user entry function 'sidon_sets' prevents it from
# appearing in the built documentation.
@cached_function
def sidon_sets_rec(N, g=1):
    r"""
    Return the set of all Sidon-`g` sets that have elements less than or equal
    to `N` without checking the arguments. This internal function should not
    be call directly by user.

    TESTS::

        sage: from sage.combinat.sidon_sets import sidon_sets_rec
        sage: sorted(sidon_sets_rec(3,2), key=str)
        [{1, 2, 3}, {1, 2}, {1, 3}, {1}, {2, 3}, {2}, {3}, {}]
    """
    if N == 1:
        return Set([Set([]), Set([1])])

    pre_sidons = sidon_sets(N - 1, g)
    sidons = set(pre_sidons)
    for psid in pre_sidons:
        psid_shift = Set([n - 1 for n in psid if n != 1] + [N - 1])
        if psid_shift not in pre_sidons:
            continue

        if 1 not in psid:
            add_sid = True
        else:
            add_sid = True
            Np1_count = 0
            for n in psid:
                if N + 1 - n in psid and 2 * n <= N + 1:
                    Np1_count += 1
                    if Np1_count >= g:
                        add_sid = False
                        break

        if add_sid:
            sidons.add(Set(psid.list() + [N]))

    return Set(sidons)



# sage_setup: distribution = sagemath-combinat
"""
Characteristic species
"""
# ****************************************************************************
#       Copyright (C) 2008 Mike Hansen <mhansen@gmail.com>,
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#    This code is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    General Public License for more details.
#
#  The full text of the GPL is available at:
#
#                  http://www.gnu.org/licenses/
# ****************************************************************************

from .species import GenericCombinatorialSpecies
from sage.arith.misc import factorial
from .structure import GenericSpeciesStructure
from .set_species import SetSpecies
from sage.structure.unique_representation import UniqueRepresentation


class CharacteristicSpeciesStructure(GenericSpeciesStructure):
    def __repr__(self):
        """
        EXAMPLES::

            sage: F = species.CharacteristicSpecies(3)
            sage: a = F.structures([1, 2, 3]).random_element(); a
            {1, 2, 3}
            sage: F = species.SingletonSpecies()
            sage: F.structures([1]).list()
            [1]
            sage: F = species.EmptySetSpecies()
            sage: F.structures([]).list()
            [{}]
        """
        s = GenericSpeciesStructure.__repr__(self)
        if self.parent()._n == 1:
            return s[1:-1]
        else:
            return "{" + s[1:-1] + "}"

    def canonical_label(self):
        """
        EXAMPLES::

            sage: F = species.CharacteristicSpecies(3)
            sage: a = F.structures(["a", "b", "c"]).random_element(); a
            {'a', 'b', 'c'}
            sage: a.canonical_label()
            {'a', 'b', 'c'}
        """
        P = self.parent()
        rng = list(range(1, P._n + 1))
        return CharacteristicSpeciesStructure(P, self._labels, rng)

    def transport(self, perm):
        """
        Return the transport of this structure along the permutation ``perm``.

        EXAMPLES::

            sage: F = species.CharacteristicSpecies(3)
            sage: a = F.structures(["a", "b", "c"]).random_element(); a
            {'a', 'b', 'c'}
            sage: p = PermutationGroupElement((1,2))                                    # needs sage.groups
            sage: a.transport(p)                                                        # needs sage.groups
            {'a', 'b', 'c'}
        """
        return self

    def automorphism_group(self):
        """
        Return the group of permutations whose action on this structure
        leave it fixed. For the characteristic species, there is only one
        structure, so every permutation is in its automorphism group.

        EXAMPLES::

            sage: F = species.CharacteristicSpecies(3)
            sage: a = F.structures(["a", "b", "c"]).random_element(); a
            {'a', 'b', 'c'}
            sage: a.automorphism_group()                                                # needs sage.groups
            Symmetric group of order 3! as a permutation group
        """
        from sage.groups.perm_gps.permgroup_named import SymmetricGroup
        return SymmetricGroup(len(self._labels))


class CharacteristicSpecies(GenericCombinatorialSpecies, UniqueRepresentation):
    def __init__(self, n, min=None, max=None, weight=None):
        """
        Return the characteristic species of order `n`.

        This species has exactly one structure on a set of size `n`
        and no structures on sets of any other size.

        EXAMPLES::

            sage: X = species.CharacteristicSpecies(1)
            sage: X.structures([1]).list()
            [1]
            sage: X.structures([1,2]).list()
            []
            sage: X.generating_series()[0:4]
            [0, 1, 0, 0]
            sage: X.isotype_generating_series()[0:4]
            [0, 1, 0, 0]
            sage: X.cycle_index_series()[0:4]                                           # needs sage.modules
            [0, p[1], 0, 0]

            sage: F = species.CharacteristicSpecies(3)
            sage: c = F.generating_series()[0:4]
            sage: F._check()
            True
            sage: F == loads(dumps(F))
            True

        TESTS::

            sage: S1 = species.CharacteristicSpecies(1)
            sage: S2 = species.CharacteristicSpecies(1)
            sage: S3 = species.CharacteristicSpecies(2)
            sage: S4 = species.CharacteristicSpecies(2, weight=2)
            sage: S1 is S2
            True
            sage: S1 == S3
            False
        """
        self._n = n
        self._name = "Characteristic species of order %s" % n
        self._state_info = [n]
        GenericCombinatorialSpecies.__init__(self, min=min, max=max, weight=weight)

    _default_structure_class = CharacteristicSpeciesStructure

    def _structures(self, structure_class, labels):
        """
        EXAMPLES::

            sage: F = species.CharacteristicSpecies(2)
            sage: l = [1, 2, 3]
            sage: F.structures(l).list()
            []
            sage: F = species.CharacteristicSpecies(3)
            sage: F.structures(l).list()
            [{1, 2, 3}]
        """
        if len(labels) == self._n:
            yield structure_class(self, labels, range(1,self._n+1))

    _isotypes = _structures

    def _gs_term(self, base_ring):
        """
        EXAMPLES::

            sage: F = species.CharacteristicSpecies(2)
            sage: F.generating_series()[0:5]
            [0, 0, 1/2, 0, 0]
            sage: F.generating_series().count(2)
            1
        """
        return base_ring(self._weight) / base_ring(factorial(self._n))

    def _order(self):
        """
        Return the order of the generating series.

        EXAMPLES::

            sage: F = species.CharacteristicSpecies(2)
            sage: F._order()
            2
        """
        return self._n

    def _itgs_term(self, base_ring):
        """
        EXAMPLES::

            sage: F = species.CharacteristicSpecies(2)
            sage: F.isotype_generating_series()[0:5]
            [0, 0, 1, 0, 0]

        Here we test out weighting each structure by q.

        ::

            sage: R.<q> = ZZ[]
            sage: Fq = species.CharacteristicSpecies(2, weight=q)
            sage: Fq.isotype_generating_series()[0:5]
            [0, 0, q, 0, 0]
        """
        return base_ring(self._weight)

    def _cis_term(self, base_ring):
        """
        EXAMPLES::

            sage: F = species.CharacteristicSpecies(2)
            sage: g = F.cycle_index_series()                                            # needs sage.modules
            sage: g[0:5]                                                                # needs sage.modules
            [0, 0, 1/2*p[1, 1] + 1/2*p[2], 0, 0]
        """
        cis = SetSpecies(weight=self._weight).cycle_index_series(base_ring)
        return cis.coefficient(self._n)

    def _equation(self, var_mapping):
        """
        Return the right hand side of an algebraic equation satisfied by
        this species. This is a utility function called by the
        algebraic_equation_system method.

        EXAMPLES::

            sage: C = species.CharacteristicSpecies(2)
            sage: Qz = QQ['z']
            sage: R.<node0> = Qz[]
            sage: var_mapping = {'z':Qz.gen(), 'node0':R.gen()}
            sage: C._equation(var_mapping)
            z^2
        """
        return var_mapping['z']**(self._n)


#Backward compatibility
CharacteristicSpecies_class = CharacteristicSpecies


class EmptySetSpecies(CharacteristicSpecies):
    def __init__(self, min=None, max=None, weight=None):
        """
        Return the empty set species.

        This species has exactly one structure on the empty set. It is
        the same (and is implemented) as ``CharacteristicSpecies(0)``.

        EXAMPLES::

            sage: X = species.EmptySetSpecies()
            sage: X.structures([]).list()
            [{}]
            sage: X.structures([1,2]).list()
            []
            sage: X.generating_series()[0:4]
            [1, 0, 0, 0]
            sage: X.isotype_generating_series()[0:4]
            [1, 0, 0, 0]
            sage: X.cycle_index_series()[0:4]                                           # needs sage.modules
            [p[], 0, 0, 0]

        TESTS::

            sage: E1 = species.EmptySetSpecies()
            sage: E2 = species.EmptySetSpecies()
            sage: E1 is E2
            True

            sage: E = species.EmptySetSpecies()
            sage: E._check()
            True
            sage: E == loads(dumps(E))
            True
        """
        CharacteristicSpecies_class.__init__(self, 0, min=min, max=max, weight=weight)
        self._name = "Empty set species"
        self._state_info = []


#Backward compatibility
EmptySetSpecies_class = EmptySetSpecies._cached_constructor = EmptySetSpecies


class SingletonSpecies(CharacteristicSpecies):
    def __init__(self, min=None, max=None, weight=None):
        """
        Return the species of singletons.

        This species has exactly one structure on a set of size `1`. It
        is the same (and is implemented) as ``CharacteristicSpecies(1)``.

        EXAMPLES::

            sage: X = species.SingletonSpecies()
            sage: X.structures([1]).list()
            [1]
            sage: X.structures([1,2]).list()
            []
            sage: X.generating_series()[0:4]
            [0, 1, 0, 0]
            sage: X.isotype_generating_series()[0:4]
            [0, 1, 0, 0]
            sage: X.cycle_index_series()[0:4]                                           # needs sage.modules
            [0, p[1], 0, 0]

        TESTS::

            sage: S1 = species.SingletonSpecies()
            sage: S2 = species.SingletonSpecies()
            sage: S1 is S2
            True

            sage: S = species.SingletonSpecies()
            sage: S._check()
            True
            sage: S == loads(dumps(S))
            True
        """
        CharacteristicSpecies_class.__init__(self, 1, min=min, max=max, weight=weight)
        self._name = "Singleton species"
        self._state_info = []


#Backward compatibility
SingletonSpecies_class = SingletonSpecies



# sage_setup: distribution = sagemath-combinat
"""
Combinatorial triangles for posets and fans

This provides several classes and methods to convert between them.
Elements of the classes are polynomials in two variables `x` and `y`,
possibly with other parameters. The conversion methods amount to specific
invertible rational change-of-variables involving `x` and `y`.

These polynomial are called triangles because their supports, the sets
of exponents where their coefficients can be nonzero, have a triangular shape.

The M-triangle class is motivated by the generating series of Möbius numbers
for graded posets. A typical example is::

    sage: W = SymmetricGroup(4)                                                         # needs sage.groups
    sage: posets.NoncrossingPartitions(W).M_triangle()                                  # needs sage.graphs sage.groups
    M: x^3*y^3 - 6*x^2*y^3 + 6*x^2*y^2 + 10*x*y^3 - 16*x*y^2
    - 5*y^3 + 6*x*y + 10*y^2 - 6*y + 1
    sage: unicode_art(_)                                                                # needs sage.graphs sage.groups sage.modules
    ⎛ -5  10  -6   1⎞
    ⎜ 10 -16   6   0⎟
    ⎜ -6   6   0   0⎟
    ⎝  1   0   0   0⎠

The F-triangle class is motivated by the generating series of pure
simplicial complexes endowed with a distinguished facet. One can also
think about complete fans endowed with a distinguished maximal
cone. A typical example is::

    sage: # needs sage.graphs sage.modules
    sage: C = ClusterComplex(['A',3])
    sage: f = C.greedy_facet()
    sage: C.F_triangle(f)
    F: 5*x^3 + 5*x^2*y + 3*x*y^2 + y^3 + 10*x^2 + 8*x*y + 3*y^2 + 6*x + 3*y + 1
    sage: unicode_art(_)
    ⎛ 1  0  0  0⎞
    ⎜ 3  3  0  0⎟
    ⎜ 3  8  5  0⎟
    ⎝ 1  6 10  5⎠

The H-triangles are related to the F-triangles by a relationship
similar to the classical link between the f-vector and the h-vector of a
simplicial complex.

The Gamma-triangles are related to the H-triangles by an
analog of the relationship between gamma-vectors and h-vectors of flag
simplicial complexes.
"""
from sage.misc.lazy_import import lazy_import
from sage.rings.integer_ring import ZZ
from sage.rings.polynomial.polynomial_ring_constructor import PolynomialRing
from sage.structure.sage_object import SageObject

lazy_import('sage.matrix.constructor', 'matrix')


def _matrix_display(self, variables=None):
    """
    Return the 2-variable polynomial ``self`` as a matrix for display.

    INPUT:

    - ``variables`` -- (optional) choice of 2 variables

    OUTPUT:

    matrix

    EXAMPLES::

        sage: from sage.combinat.triangles_FHM import _matrix_display
        sage: x, y = PolynomialRing(QQ,['x', 'y']).gens()
        sage: _matrix_display(x**2+x*y+y**3)                                            # needs sage.modules
        [1 0 0]
        [0 0 0]
        [0 1 0]
        [0 0 1]

    With a specific choice of variables::

        sage: x, y, z = PolynomialRing(QQ,['x','y','z']).gens()
        sage: _matrix_display(x**2+z*x*y+z*y**3+z*x,[y,z])                              # needs sage.modules
        [  x   x   0   1]
        [x^2   0   0   0]
        sage: _matrix_display(x**2+z*x*y+z*y**3+z*x,[x,z])                              # needs sage.modules
        [  y^3 y + 1     0]
        [    0     0     1]
    """
    support = self.exponents()
    if variables is None:
        ring = self.parent().base_ring()
        x, y = self.parent().gens()
        ix = 0
        iy = 1
    else:
        x, y = variables
        ring = self.parent()
        all_vars = x.parent().gens()
        ix = all_vars.index(x)
        iy = all_vars.index(y)
    minx = min(u[ix] for u in support)
    maxy = max(u[iy] for u in support)
    deltax = max(u[ix] for u in support) - minx + 1
    deltay = maxy - min(u[iy] for u in support) + 1
    mat = matrix(ring, deltay, deltax)
    for u in support:
        ex = u[ix]
        ey = u[iy]
        mat[maxy - ey, ex - minx] = self.coefficient({x: ex, y: ey})
    return mat


class Triangle(SageObject):
    """
    Common class for different kinds of triangles.

    This serves as a base class for F-triangles, H-triangles, M-triangles
    and Gamma-triangles.

    The user should use these subclasses directly.

    The input is a polynomial in two variables. One can also give a
    polynomial with more variables and specify two chosen variables.

    EXAMPLES::

        sage: from sage.combinat.triangles_FHM import Triangle
        sage: x, y = polygens(ZZ, 'x,y')
        sage: ht = Triangle(1+4*x+2*x*y)
        sage: unicode_art(ht)                                                           # needs sage.modules
        ⎛0 2⎞
        ⎝1 4⎠
    """

    def __init__(self, poly, variables=None):
        """
        EXAMPLES::

            sage: from sage.combinat.triangles_FHM import Triangle
            sage: x, y = polygens(ZZ, 'x,y')
            sage: ht = Triangle(1+2*x*y)
            sage: unicode_art(ht)                                                       # needs sage.modules
            ⎛0 2⎞
            ⎝1 0⎠
        """
        if variables is None:
            self._vars = poly.parent().gens()
        else:
            self._vars = variables
        self._poly = poly
        self._n = max(self._poly.degree(v) for v in self._vars)

    def _ascii_art_(self):
        """
        Return the ascii-art representation (as a matrix).

        EXAMPLES::

            sage: from sage.combinat.triangles_FHM import H_triangle
            sage: x, y = polygens(ZZ, 'x,y')
            sage: ht = H_triangle(1+2*x*y)
            sage: ascii_art(ht)                                                         # needs sage.modules
            [0 2]
            [1 0]
        """
        return self.matrix()._ascii_art_()

    def _unicode_art_(self):
        """
        Return the unicode representation (as a matrix).

        EXAMPLES::

            sage: from sage.combinat.triangles_FHM import H_triangle
            sage: x, y = polygens(ZZ, 'x,y')
            sage: ht = H_triangle(1+2*x*y)
            sage: unicode_art(ht)                                                       # needs sage.modules
            ⎛0 2⎞
            ⎝1 0⎠
        """
        return self.matrix()._unicode_art_()

    def _repr_(self) -> str:
        """
        Return the string representation (as a polynomial).

        EXAMPLES::

            sage: from sage.combinat.triangles_FHM import H_triangle
            sage: x, y = polygens(ZZ, 'x,y')
            sage: ht = H_triangle(1+2*x*y)
            sage: ht
            H: 2*x*y + 1
        """
        return self._prefix + ": " + repr(self._poly)

    def _latex_(self):
        r"""
        Return the LaTeX representation (as a matrix).

        EXAMPLES::

            sage: from sage.combinat.triangles_FHM import H_triangle
            sage: x, y = polygens(ZZ, 'x,y')
            sage: ht = H_triangle(1+2*x*y)
            sage: latex(ht)                                                             # needs sage.modules
            \left(\begin{array}{rr}
            0 & 2 \\
            1 & 0
            \end{array}\right)
        """
        return self.matrix()._latex_()

    def __eq__(self, other) -> bool:
        """
        Test for equality.

        EXAMPLES::

            sage: from sage.combinat.triangles_FHM import H_triangle
            sage: x, y = polygens(ZZ, 'x,y')
            sage: h1 = H_triangle(1+2*x*y)
            sage: h2 = H_triangle(1+3*x*y)
            sage: h1 == h1
            True
            sage: h1 == h2
            False
        """
        if isinstance(other, Triangle):
            return self._poly == other._poly
        return self._poly == other

    def __ne__(self, other) -> bool:
        """
        Test for unequality.

        EXAMPLES::

            sage: from sage.combinat.triangles_FHM import H_triangle
            sage: x, y = polygens(ZZ, 'x,y')
            sage: h1 = H_triangle(1+2*x*y)
            sage: h2 = H_triangle(1+3*x*y)
            sage: h1 != h1
            False
            sage: h1 != h2
            True
        """
        return not self == other

    def __call__(self, *args):
        """
        Return the evaluation (as a polynomial).

        EXAMPLES::

            sage: from sage.combinat.triangles_FHM import H_triangle
            sage: x, y = polygens(ZZ, 'x,y')
            sage: h = H_triangle(1+3*x*y)
            sage: h(4,5)
            61
        """
        return self._poly(*args)

    def __getitem__(self, *args):
        """
        Return some coefficient.

        EXAMPLES::

            sage: from sage.combinat.triangles_FHM import H_triangle
            sage: x, y = polygens(ZZ, 'x,y')
            sage: h = H_triangle(1+2*x+3*x*y)
            sage: h[1,1]
            3
        """
        return self._poly.__getitem__(*args)

    def __hash__(self):
        """
        Return the hash value.

        EXAMPLES::

            sage: from sage.combinat.triangles_FHM import H_triangle
            sage: x, y = polygens(ZZ, 'x,y')
            sage: h = H_triangle(1+2*x*y)
            sage: g = H_triangle(1+2*x*y)
            sage: hash(h) == hash(g)
            True
        """
        return hash(self._poly)

    def matrix(self):
        """
        Return the associated matrix for display.

        EXAMPLES::

            sage: from sage.combinat.triangles_FHM import H_triangle
            sage: x, y = polygens(ZZ, 'x,y')
            sage: h = H_triangle(1+2*x*y)
            sage: h.matrix()                                                            # needs sage.modules
            [0 2]
            [1 0]
        """
        return _matrix_display(self._poly, variables=self._vars)

    def polynomial(self):
        """
        Return the triangle as a bare polynomial.

        EXAMPLES::

            sage: from sage.combinat.triangles_FHM import H_triangle
            sage: x, y = polygens(ZZ, 'x,y')
            sage: h = H_triangle(1+2*x*y)
            sage: h.polynomial()
            2*x*y + 1
        """
        return self._poly

    def truncate(self, d):
        """
        Return the truncated triangle.

        INPUT:

        - ``d`` -- integer

        As a polynomial, this means that all monomials with a power
        of either `x` or `y` greater than or equal to ``d`` are dismissed.

        EXAMPLES::

            sage: from sage.combinat.triangles_FHM import H_triangle
            sage: x, y = polygens(ZZ, 'x,y')
            sage: h = H_triangle(1+2*x*y)
            sage: h.truncate(2)
            H: 2*x*y + 1
        """
        p = self._poly
        for v in self._vars:
            p = p.truncate(v, d)
        return self.__class__(p, self._vars)


class M_triangle(Triangle):
    """
    Class for the M-triangles.

    This is motivated by generating series of Möbius numbers of graded posets.

    EXAMPLES::

        sage: x, y = polygens(ZZ, 'x,y')
        sage: P = Poset({2: [1]})                                                       # needs sage.graphs
        sage: P.M_triangle()                                                            # needs sage.graphs
        M: x*y - y + 1
    """
    _prefix = 'M'

    def dual(self):
        """
        Return the dual M-triangle.

        This is the M-triangle of the dual poset, hence an involution.

        When seen as a matrix, this performs a symmetry with respect to the
        northwest-southeast diagonal.

        EXAMPLES::

            sage: from sage.combinat.triangles_FHM import  M_triangle
            sage: x, y = polygens(ZZ, 'x,y')
            sage: mt = M_triangle(x*y - y + 1)
            sage: mt.dual() == mt
            True
        """
        x, y = self._vars
        n = self._n
        A = self._poly.parent()

        dict_dual = {(n - dy, n - dx): coeff
                     for (dx, dy), coeff in self._poly.monomial_coefficients().items()}
        return M_triangle(A(dict_dual), variables=(x, y))

    def transmute(self):
        """
        Return the image of ``self`` by an involution.

        OUTPUT: another M-triangle

        The involution is defined by converting to an H-triangle,
        transposing the matrix, and then converting back to an M-triangle.

        EXAMPLES::

            sage: from sage.combinat.triangles_FHM import  M_triangle
            sage: x, y = polygens(ZZ, 'x,y')
            sage: nc3 = x^2*y^2 - 3*x*y^2 + 3*x*y + 2*y^2 - 3*y + 1
            sage: m = M_triangle(nc3)
            sage: m2 = m.transmute(); m2                                                # needs sage.libs.flint
            M: 2*x^2*y^2 - 3*x*y^2 + 2*x*y + y^2 - 2*y + 1
            sage: m2.transmute() == m                                                   # needs sage.libs.flint
            True
        """
        return self.h().transpose().m()

    def h(self):
        """
        Return the associated H-triangle.

        EXAMPLES::

            sage: from sage.combinat.triangles_FHM import M_triangle
            sage: x, y = polygens(ZZ,'x,y')
            sage: M_triangle(1-y+x*y).h()
            H: x*y + 1

        TESTS::

            sage: h = polygen(ZZ, 'h')
            sage: x, y = polygens(h.parent(),'x,y')
            sage: mt = x**2*y**2+(-2*h+2)*x*y**2+(2*h-2)*x*y+(2*h-3)*y**2+(-2*h+2)*y+1
            sage: M_triangle(mt, [x,y]).h()
            H: x^2*y^2 + 2*x*y + (2*h - 4)*x + 1
        """
        x, y = self._vars
        n = self._n
        step = self._poly.subs({x: y / (y - 1),
                                y: (y - 1) * x / (1 + (y - 1) * x)})
        step *= (1 + (y - 1) * x)**n
        polyh = step.numerator()
        return H_triangle(polyh, variables=(x, y))

    def f(self):
        """
        Return the associated F-triangle.

        EXAMPLES::

            sage: from sage.combinat.triangles_FHM import M_triangle
            sage: x, y = polygens(ZZ,'x,y')
            sage: M_triangle(1-y+x*y).f()
            F: x + y + 1

        TESTS::

            sage: h = polygen(ZZ, 'h')
            sage: x, y = polygens(h.parent(),'x,y')
            sage: mt = x**2*y**2+(-2*h+2)*x*y**2+(2*h-2)*x*y+(2*h-3)*y**2+(-2*h+2)*y+1
            sage: M_triangle(mt, [x,y]).f()
            F: (2*h - 3)*x^2 + 2*x*y + y^2 + (2*h - 2)*x + 2*y + 1
        """
        return self.h().f()


class H_triangle(Triangle):
    """
    Class for the H-triangles.
    """
    _prefix = 'H'

    def transpose(self):
        """
        Return the transposed H-triangle.

        OUTPUT: another H-triangle

        This operation is an involution.  When seen as a matrix, it
        performs a symmetry with respect to the northwest-southeast
        diagonal.

        EXAMPLES::

            sage: from sage.combinat.triangles_FHM import H_triangle
            sage: x, y = polygens(ZZ,'x,y')
            sage: H_triangle(1+x*y).transpose()
            H: x*y + 1
            sage: H_triangle(x^2*y^2 + 2*x*y + x + 1).transpose()
            H: x^2*y^2 + x^2*y + 2*x*y + 1
        """
        x, y = self._vars
        n = self._n
        A = self._poly.parent()

        dict_dual = {(n - dy, n - dx): coeff
                     for (dx, dy), coeff in self._poly.monomial_coefficients().items()}
        return H_triangle(A(dict_dual), variables=(x, y))

    def m(self):
        """
        Return the associated M-triangle.

        EXAMPLES::

            sage: from sage.combinat.triangles_FHM import H_triangle
            sage: h = polygen(ZZ, 'h')
            sage: x, y = polygens(h.parent(),'x,y')
            sage: ht = H_triangle(x^2*y^2 + 2*x*y + 2*x*h - 4*x + 1, variables=[x,y])
            sage: ht.m()
            M: x^2*y^2 + (-2*h + 2)*x*y^2 + (2*h - 2)*x*y
            + (2*h - 3)*y^2 + (-2*h + 2)*y + 1
        """
        x, y = self._vars
        n = self._n
        step = self._poly.subs({x: (x - 1) * y / (1 - y),
                                y: x / (x - 1)}) * (1 - y)**n
        polym = step.numerator()
        return M_triangle(polym, variables=(x, y))

    def f(self):
        """
        Return the associated F-triangle.

        EXAMPLES::

            sage: from sage.combinat.triangles_FHM import H_triangle
            sage: x, y = polygens(ZZ,'x,y')
            sage: H_triangle(1+x*y).f()
            F: x + y + 1
            sage: H_triangle(x^2*y^2 + 2*x*y + x + 1).f()
            F: 2*x^2 + 2*x*y + y^2 + 3*x + 2*y + 1
            sage: flo = H_triangle(1+4*x+2*x**2+x*y*(4+8*x)+
            ....:   x**2*y**2*(6+4*x)+4*(x*y)**3+(x*y)**4).f(); flo
            F: 7*x^4 + 12*x^3*y + 10*x^2*y^2 + 4*x*y^3 + y^4 + 20*x^3
            + 28*x^2*y + 16*x*y^2 + 4*y^3 + 20*x^2 + 20*x*y
            + 6*y^2 + 8*x + 4*y + 1
            sage: flo(-1-x,-1-y) == flo
            True

        TESTS::

            sage: x,y,h = polygens(ZZ,'x,y,h')
            sage: ht = x^2*y^2 + 2*x*y + 2*x*h - 4*x + 1
            sage: H_triangle(ht,[x,y]).f()
            F: 2*x^2*h - 3*x^2 + 2*x*y + y^2 + 2*x*h - 2*x + 2*y + 1
        """
        x, y = self._vars
        n = self._n
        step1 = self._poly.subs({x: x / (1 + x), y: y}) * (x + 1)**n
        step2 = step1.subs({x: x, y: y / x})
        polyf = step2.numerator()
        return F_triangle(polyf, variables=(x, y))

    def gamma(self):
        """
        Return the associated Gamma-triangle.

        In some cases, this is a more condensed way to encode
        the same amount of information.

        EXAMPLES::

            sage: from sage.combinat.triangles_FHM import H_triangle
            sage: x, y = polygen(ZZ,'x,y')
            sage: ht = x**2*y**2 + 2*x*y + x + 1
            sage: H_triangle(ht).gamma()
            Γ: y^2 + x

            sage: W = SymmetricGroup(5)                                                 # needs sage.groups
            sage: P = posets.NoncrossingPartitions(W)                                   # needs sage.graphs sage.groups
            sage: P.M_triangle().h().gamma()                                            # needs sage.graphs sage.groups
            Γ: y^4 + 3*x*y^2 + 2*x^2 + 2*x*y + x
        """
        x, y = self._vars
        n = self._n
        remain = self._poly
        gamma = x.parent().zero()
        for k in range(n, -1, -1):
            step = remain.coefficient({x: k})
            gamma += x**(n - k) * step
            remain -= x**(n - k) * step.homogenize(x)(x=1 + x, y=1 + x * y)
        return Gamma_triangle(gamma, variables=(x, y))

    def vector(self):
        """
        Return the h-vector as a polynomial in one variable.

        This is obtained by letting `y=1`.

        EXAMPLES::

            sage: from sage.combinat.triangles_FHM import H_triangle
            sage: x, y = polygen(ZZ,'x,y')
            sage: ht = x**2*y**2 + 2*x*y + x + 1
            sage: H_triangle(ht).vector()
            x^2 + 3*x + 1
        """
        x, y = self._vars
        anneau = PolynomialRing(ZZ, "x")
        return anneau(self._poly.subs({y: 1}))


class F_triangle(Triangle):
    """
    Class for the F-triangles.
    """
    _prefix = 'F'

    def h(self):
        """
        Return the associated H-triangle.

        EXAMPLES::

            sage: from sage.combinat.triangles_FHM import F_triangle
            sage: x,y = polygens(ZZ,'x,y')
            sage: ft = F_triangle(1+x+y)
            sage: ft.h()
            H: x*y + 1

        TESTS::

            sage: h = polygen(ZZ, 'h')
            sage: x, y = polygens(h.parent(),'x,y')
            sage: ft = 1+2*y+(2*h-2)*x+y**2+2*x*y+(2*h-3)*x**2
            sage: F_triangle(ft, [x,y]).h()
            H: x^2*y^2 + 2*x*y + (2*h - 4)*x + 1
        """
        x, y = self._vars
        n = self._n
        step = (1 - x)**n * self._poly.subs({x: x / (1 - x),
                                             y: x * y / (1 - x)})
        polyh = step.numerator()
        return H_triangle(polyh, variables=(x, y))

    def m(self):
        """
        Return the associated M-triangle.

        EXAMPLES::

            sage: from sage.combinat.triangles_FHM import H_triangle
            sage: x, y = polygens(ZZ,'x,y')
            sage: H_triangle(1+x*y).f()
            F: x + y + 1
            sage: _.m()
            M: x*y - y + 1

            sage: H_triangle(x^2*y^2 + 2*x*y + x + 1).f()
            F: 2*x^2 + 2*x*y + y^2 + 3*x + 2*y + 1
            sage: _.m()
            M: x^2*y^2 - 3*x*y^2 + 3*x*y + 2*y^2 - 3*y + 1

        TESTS::

            sage: p = 1+4*x+2*x**2+x*y*(4+8*x)
            sage: p += x**2*y**2*(6+4*x)+4*(x*y)**3+(x*y)**4
            sage: flo = H_triangle(p).f(); flo
            F: 7*x^4 + 12*x^3*y + 10*x^2*y^2 + 4*x*y^3 + y^4 + 20*x^3
            + 28*x^2*y + 16*x*y^2 + 4*y^3 + 20*x^2 + 20*x*y
            + 6*y^2 + 8*x + 4*y + 1
            sage: flo.m()
            M: x^4*y^4 - 8*x^3*y^4 + 8*x^3*y^3 + 20*x^2*y^4 - 36*x^2*y^3
            - 20*x*y^4 + 16*x^2*y^2 + 48*x*y^3 + 7*y^4 - 36*x*y^2 - 20*y^3
            + 8*x*y + 20*y^2 - 8*y + 1

            sage: from sage.combinat.triangles_FHM import F_triangle
            sage: h = polygen(ZZ, 'h')
            sage: x, y = polygens(h.parent(),'x,y')
            sage: ft = F_triangle(1+2*y+(2*h-2)*x+y**2+2*x*y+(2*h-3)*x**2,(x,y))
            sage: ft.m()
            M: x^2*y^2 + (-2*h + 2)*x*y^2 + (2*h - 2)*x*y
            + (2*h - 3)*y^2 + (-2*h + 2)*y + 1
        """
        x, y = self._vars
        n = self._n
        step = self._poly.subs({x: y * (x - 1) / (1 - x * y),
                                y: x * y / (1 - x * y)})
        step *= (1 - x * y)**n
        polym = step.numerator()
        return M_triangle(polym, variables=(x, y))

    def parabolic(self):
        """
        Return a parabolic version of the F-triangle.

        This is obtained by replacing the variable `y` by `y-1`.

        EXAMPLES::

            sage: from sage.combinat.triangles_FHM import H_triangle
            sage: x, y = polygens(ZZ,'x,y')
            sage: H_triangle(1+x*y).f()
            F: x + y + 1
            sage: _.parabolic()
            F: x + y

        TESTS::

            sage: a, b = polygens(ZZ,'a,b')
            sage: H_triangle(1+a*b).f()
            F: a + b + 1
            sage: _.parabolic()
            F: a + b
        """
        x, y = self._vars
        polyf = self._poly.subs({y: y - 1})
        return F_triangle(polyf, variables=(x, y))

    def vector(self):
        """
        Return the f-vector as a polynomial in one variable.

        This is obtained by letting `y=x`.

        EXAMPLES::

            sage: from sage.combinat.triangles_FHM import F_triangle
            sage: x, y = polygen(ZZ,'x,y')
            sage: ft = 2*x^2 + 2*x*y + y^2 + 3*x + 2*y + 1
            sage: F_triangle(ft).vector()
            5*x^2 + 5*x + 1
        """
        x, y = self._vars
        anneau = PolynomialRing(ZZ, "x")
        nx = anneau.gen()
        return anneau(self._poly.subs({x: nx, y: nx}))


class Gamma_triangle(Triangle):
    """
    Class for the Gamma-triangles.
    """
    _prefix = 'Γ'

    def h(self):
        r"""
        Return the associated H-triangle.

        The transition between Gamma-triangles and H-triangles is defined by

        .. MATH::

            H(x,y) = (1+x)^d \sum_{0\leq i; 0\leq j \leq d-2i} \gamma_{i,j}
            \left(\frac{x}{(1+x)^2}\right)^i \left(\frac{1+xy}{1+x}\right)^j

        EXAMPLES::

            sage: from sage.combinat.triangles_FHM import Gamma_triangle
            sage: x, y = polygen(ZZ,'x,y')
            sage: g = y**2 + x
            sage: Gamma_triangle(g).h()
            H: x^2*y^2 + 2*x*y + x + 1

            sage: a, b = polygen(ZZ, 'a, b')
            sage: x, y = polygens(a.parent(),'x,y')
            sage: g = Gamma_triangle(y**3+a*x*y+b*x,(x,y))
            sage: hh = g.h()
            sage: hh.gamma() == g
            True
        """
        x, y = self._vars
        n = self._n
        resu = (1 + x)**n * self._poly(x=x / (1 + x)**2,
                                       y=(1 + x * y) / (1 + x))
        polyh = resu.numerator()
        return H_triangle(polyh, variables=(x, y))

    def vector(self):
        """
        Return the gamma-vector as a polynomial in one variable.

        This is obtained by letting `y=1`.

        EXAMPLES::

            sage: from sage.combinat.triangles_FHM import Gamma_triangle
            sage: x, y = polygen(ZZ,'x,y')
            sage: gt = y**2 + x
            sage: Gamma_triangle(gt).vector()
            x + 1
        """
        anneau = PolynomialRing(ZZ, 'x')
        return anneau(self._poly(y=1))
